/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml.Serialization;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;
using Microsoft.SharePoint.WebControls; // for SPControl
using System.Text; // for StringBuilder
using PREPWorkflow;
using System.Collections; // for ArrayList
using System.Configuration; // for picking up values from machine.config
using System.Diagnostics; // For Tracing
//using System.Security;
using System.Security.Policy;

namespace PREPWebPartLibrary20
{
	/// <summary>
	/// The PREP Matrix Web Part. This is the main user interface to PREP. It contains one grid for each 
	/// PREP assignment. Each grid contains a row and a column for each student who belongs to the assignment. 
	/// The boxes on the grid contain references to the students documents and tasks. 
	/// </summary>
	[DefaultProperty("Text"),
		ToolboxData("<{0}:PREPMatrixWebPart20 runat=server></{0}:PREPMatrixWebPart20>"),
		XmlRoot(Namespace="PREPWebPartLibrary20")]
	public class PREPMatrixWebPart20 : Microsoft.SharePoint.WebPartPages.WebPart
	{
		
		private bool traceEnabled = false;
		
		private string prepDocLibraryName = ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"];

		private string prepTaskListName = ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"];

		// PREPWorkflow.Username, PREPWorkflow.Domain, and PREPWorkflow.Password are stored in machine.config
		private string privelegedUserID = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Username"]);
		private string domain = ConfigurationSettings.AppSettings["PREPWorkflow.Domain"];
		private string password = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Password"]);

		/// <summary>
		/// List of constants which represent file names of images used in the PREP Matrix.
		/// </summary>
		public struct PREPImages
		{
			/// <summary>
			/// Image that represents an Ideas Task on the PREP Matrix.
			/// </summary>
			public static string IdeasTask = "ideas_task.gif";
			/// <summary>
			/// Image that represents the logged-in user's clickable Ideas Task on the PREP Matrix.
			/// </summary>
			public static string MyIdeasTask = "my_ideas_task.gif";
			/// <summary>
			/// Image that represents a document on the PREP Matrix.
			/// </summary>
			public static string Document = "doc.gif";
			/// <summary>
			/// Image that represents a task that has not been started on the PREP Matrix.
			/// </summary>
			public static string TaskNotStarted = "black.gif";
			/// <summary>
			/// Image that represents a task that is in progress on the PREP Matrix.
			/// </summary>
			public static string TaskInProgress = "yellow.gif";
			/// <summary>
			/// Image that represents the logged-in user's clickable task on the PREP Matrix. This one has a status of Not Started.
			/// </summary>
			public static string MyTaskNotStarted = "go_green.gif";
			/// <summary>
			/// Image that represents the logged-in user's clickable task on the PREP Matrix. This one has a status of In Progress.
			/// </summary>
			public static string MyTaskInProgress = "go_yellow.gif";
			/// <summary>
			/// Image that represents a completed task. This will be clickable, and lead to a Review Document.
			/// </summary>
			public static string TaskCompleted = "finished.gif";
		}

		/// <summary>
		/// If true, the Web Part will write trace information to the Trace Writers. 
		/// This trace information may be used by the TraceInfo Web Part.
		/// </summary>
		[Browsable(true),
			Category("Settings"),
			DefaultValue(false),
			WebPartStorage(Storage.Personal),
			FriendlyName("Enable Trace"),
			Description("Text Property")]
		public bool TraceEnabled
		{
			get
			{
				return traceEnabled;
			}

			set
			{
				traceEnabled = value;
			}
		}

		/// <summary>
		/// Override OnInit in order to set the title of the web part
		/// </summary>
		/// <param name="e"></param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit (e);
			this.Title = "PREP Matrix";
		}
	
		/// <summary>
		/// Render this Web Part to the output parameter specified.
		/// </summary>
		/// <param name="output"> The HTML writer to write out to </param>
		protected override void RenderWebPart(HtmlTextWriter output)
		{
			try
			{
				output.Write(PREPMatrix());
			}
			catch (Exception ex)
			{
				Utilities.LogError("Cannot write PREPMatrix()", ex);
			}
		}

		/// <summary>
		/// The PREP Matrix is an HTML table having one row per user, and one column per user.
		/// The intersection of a user's row and column represents that user's project submission.
		/// All other intersections represent a user's task to review another user's work.
		/// </summary>
		/// <returns></returns>
		private string PREPMatrix()
		{
			
			// Get the logged in user's login name
			string loggedInUserLoginName = Utilities.GetLoginName(Context.User.Identity.Name);
			
			// Get the current web context
			SPWeb PREPWeb = SPControl.GetContextWeb(Context);

			// Get the list object that represents the document library
			SPList prepDocumentList = PREPWeb.Lists[prepDocLibraryName];

			// Populate an ArrayList of PREP Task objects
			ArrayList prepTasks = PREPTask.PREPTask_Collection(PREPWeb.Lists[prepTaskListName]);

			StringBuilder prepMatrix = new StringBuilder();

			string statusCode;

			StringBuilder docImage;
			
			// Loop through Projects. There will be one table (i.e. PREP Matrix) per project).
			ArrayList prepAssignments = PREPAssignment.PREPAssignment_Collection(PREPWeb);
			foreach(PREPAssignment prepAssignment in prepAssignments)
			{
			
				// Start building the table
				prepMatrix.Append(Environment.NewLine);
				prepMatrix.Append(HTMLTools.Comment("-=-=-= Begin PREP Matrix =-=-=-"));
				prepMatrix.Append(HTMLTools.StartTable("style=\"BORDER-RIGHT: black 1px solid; BORDER-TOP: black 1px solid; BORDER-LEFT: black 1px solid; WIDTH: 95%; BORDER-BOTTOM: black 1px solid; BORDER-COLLAPSE: collapse\" borderColor=black cellSpacing=0 rules=all align=center border=1"));
				
				// Project Header Row
				int numCols = PREPWeb.Users.Count + 1; // one column per user, plus the label column
				prepMatrix.Append(HTMLTools.StartRow("style=\"FONT-WEIGHT: bold; BACKGROUND-COLOR: gold\""));
				prepMatrix.Append(HTMLTools.StartCol("colspan=\"" + numCols.ToString() + "\"" ) + prepAssignment.AssignmentName + HTMLTools.EndCol());
				prepMatrix.Append(HTMLTools.EndRow());

				// Header Row
				prepMatrix.Append(HTMLTools.StartRow("style=\"FONT-WEIGHT: bold; BACKGROUND-COLOR: gold\""));
				prepMatrix.Append(HTMLTools.StartCol() + "Ideas" + HTMLTools.EndCol());

				// Loop through users
				// Because of a SharePoint limitation, the SPRole object is only available to site administrators,
				// even if you are using impersonation.
				//SPRole assignmentRole = PREPWeb.Roles[prepAssignment.AssignmentName];
				//SPUserCollection assignmentUsers = assignmentRole.Users; //PREPWeb.Users;
				//foreach (SPUser user in assignmentUsers)
				PREPRole assignmentRole = new PREPRole(PREPWeb, prepAssignment.AssignmentName);
				ArrayList assignmentUsers = assignmentRole.Users;
				foreach (SPUserInfo user in assignmentUsers)
				{
					// The header row will have one column header for each user
					prepMatrix.Append(HTMLTools.StartCol());
					prepMatrix.Append(Utilities.GetLoginName(user.LoginName));
					prepMatrix.Append(HTMLTools.EndCol());
				}
				prepMatrix.Append(HTMLTools.EndRow());

				// User Rows. There will be one row for each user.
				int colorSwitch = 0;
				string imageName;
				string altTag;
				//foreach (SPUser rowUser in assignmentUsers)
				foreach(SPUserInfo rowUser in assignmentUsers)
				{
					// alternate white and grey rows
					if (colorSwitch == 0)
					{ 
						prepMatrix.Append(HTMLTools.StartRow("vAlign=\"center\""));
						colorSwitch = 1;
					}
					else
					{ 
						prepMatrix.Append(HTMLTools.StartRow("vAlign=\"center\" style=\"BACKGROUND-COLOR: gainsboro\""));
						colorSwitch = 0;
					}
					// Column 1 - the user name on the left side under the heading "Ideas"

					string rowUserLoginName = Utilities.GetLoginName(rowUser.LoginName);

					// The user name "label" column at the far left
					prepMatrix.Append(HTMLTools.StartCol());
					prepMatrix.Append(rowUserLoginName);
					prepMatrix.Append(HTMLTools.EndCol());
				
					// User columns. For each User row, there will be one column for each user.
					//foreach (SPUser colUser in assignmentUsers)
					foreach (SPUserInfo colUser in assignmentUsers)
					{
				
						string colUserLoginName = Utilities.GetLoginName(colUser.LoginName);
						
						//Trace
						if(traceEnabled)
						{
							Trace.WriteLine("ALL USERS " + "rowUserLoginName=" + rowUserLoginName + " colUserLoginName="+colUserLoginName + " rowUser.LoginName="+rowUser.LoginName + " colUser.LoginName="+colUser.LoginName, "PREPMatrix");
						}
						
						// Start User column
						prepMatrix.Append(HTMLTools.Comment("Begin User column"));
						prepMatrix.Append(HTMLTools.StartCol("align=\"center\""));

						// If this is an intersection of the current user's row and column, put a link to the document
						if(rowUser.LoginName == colUser.LoginName)
						{
							//Trace
							if(traceEnabled)
							{
								Trace.WriteLine("ROW==COL " + "rowUserLoginName=" + rowUserLoginName + " colUserLoginName="+colUserLoginName + " rowUser.LoginName="+rowUser.LoginName + " colUser.LoginName="+colUser.LoginName, "PREPMatrix");
							}
						
							//first, see if there is an "ideas" task. This is a task obligating this user to create and upload an ideas document.
							PREPTask ideasTask = PREPTask.GetIdeasTask(prepTasks, prepAssignment.AssignmentName, rowUserLoginName);
							if(ideasTask.Exists)
							{
								//Trace
								if(traceEnabled)
								{
									StringBuilder traceInfo = new StringBuilder();
									traceInfo.Append("ideasTask.AssignmentName=" + ideasTask.AssignmentName);
									traceInfo.Append(" ideasTask.AssignedTo=" +ideasTask.AssignedTo);
									traceInfo.Append(" ideasTask.DocumentAuthor" + ideasTask.DocumentAuthor);
									traceInfo.Append(" ideasTask.DocumentName" + ideasTask.DocumentName);
									traceInfo.Append(" ideasTask.Title" + ideasTask.Title);
									traceInfo.Append(" ideasTask.Status" + ideasTask.Status);
									Trace.WriteLine(traceInfo.ToString(), "PREPMatrix");
								}

								if(ideasTask.Status == PREPFields.NotStarted)
								{
									altTag = ideasTask.Title + " Status: " + ideasTask.Status;

									// If this task belongs to the logged-in user, make the task a link to upload a document
									if(rowUserLoginName == loggedInUserLoginName)
									{
										
										imageName = PREPImages.MyIdeasTask;
										altTag += ". CLICK HERE TO UPLOAD.";
										// Document image link.
										docImage = new StringBuilder();

										//http://sharepointhost.school.edu/_layouts/PrepUploadDownload.aspx?fileName=my_ideas.doc&assignmentName=TheAssignment&ideasAuthorName=my_user_id&prepAction=upload
										docImage.Append("<A href=" + PREPWeb.Url + "/_layouts/PrepUploadDownload.aspx?assignmentName=" + prepAssignment.AssignmentName + "&ideasAuthorName=" + loggedInUserLoginName + "&prepAction=upload" + ">");
									
										//display portion of the anchor: an image
										docImage.Append(HTMLTools.Image(imageName, altTag));
										docImage.Append("</a>" + Environment.NewLine);
										prepMatrix.Append(docImage.ToString());
									}
									else
									{
										// For everyone else, just display the task
										imageName = PREPImages.IdeasTask;
										prepMatrix.Append(HTMLTools.Image(imageName, altTag));
									}
								}
								else //if(ideasTask.Status == PREPFields.NotStarted)
								{
									// if the task is in progress, look for a document
									PREPDocument ideasDoc = PREPDocument.GetIdeasDoc(PREPWeb, prepAssignment.AssignmentName, colUserLoginName);
									if(ideasDoc.Exists)
									{
										// Document image link.
										docImage = new StringBuilder();
										imageName = PREPImages.Document;
										//altTag = ideasDoc.FileName + ": " + ideasDoc.IdeasAuthorLogin + "'s ideas.";
										// rowUser.Name will provide the full name, whereas ideasDoc.IdeasAuthorLogin just shows the login name - which may make it hard to guess who the actual user is.
										altTag = ideasDoc.FileName + ": " + rowUser.Name + "'s ideas.";
										
										//"<A href=" + PREPWeb.Url + "/_layouts/PrepUploadDownload.aspx?fileName=" + Context.Server.HtmlEncode(ideasDoc.FileName) + "&assignmentName=" + prepAssignment.AssignmentName + "&ideasAuthorName=" + colUserLoginName + "&prepAction=download" + ">"
										string docUrl = SPEncode.HtmlEncode((PREPWeb.Url + "/_layouts/PrepUploadDownload.aspx?fileName=" + SPEncode.UrlEncode(ideasDoc.FileName) + "&assignmentName=" + prepAssignment.AssignmentName + "&ideasAuthorName=" + colUserLoginName + "&prepAction=download"));
										
										//display portion of the anchor is an image
										docImage.Append("<A href=\"" + docUrl + "\">" + HTMLTools.Image(imageName, altTag) + "</A>" + Environment.NewLine);
										prepMatrix.Append(docImage.ToString());
									}
								}
							}
						} //(rowUser.LoginName == colUser.LoginName)
						else
						{
							//Trace
							if(traceEnabled)
							{
								Trace.WriteLine("ROW!=COL " + "rowUserLoginName=" + rowUserLoginName + " colUserLoginName="+colUserLoginName + " rowUser.LoginName="+rowUser.LoginName + " colUser.LoginName="+colUser.LoginName, "PREPMatrix");
							}
							
							// If this is not an intersection of the current user's row and column, look for a task in which the 
							// current row's user is assigned to review the current column's user's ideas
							PREPTask reviewTask = PREPTask.GetReviewTask(prepTasks, prepAssignment.AssignmentName, colUserLoginName, rowUserLoginName);

							if(reviewTask.Exists)
							{
								//Trace
								if(traceEnabled)
								{
									StringBuilder traceInfo = new StringBuilder();
									traceInfo.Append("reviewTask.AssignmentName=" + reviewTask.AssignmentName);
									traceInfo.Append(" reviewTask.AssignedTo=" +reviewTask.AssignedTo);
									traceInfo.Append(" reviewTask.DocumentAuthor" + reviewTask.DocumentAuthor);
									traceInfo.Append(" reviewTask.DocumentName" + reviewTask.DocumentName);
									traceInfo.Append(" reviewTask.Title" + reviewTask.Title);
									traceInfo.Append(" reviewTask.Status" + reviewTask.Status);
									Trace.WriteLine(traceInfo.ToString(), "PREPMatrix");
								}
								
								statusCode = reviewTask.Status;

								// Document image link.
								docImage = new StringBuilder();

								// create a link, with a different icon, if this row belongs to the logged-in user
								if(rowUserLoginName == loggedInUserLoginName)
								{
									switch (statusCode)
									{
										case PREPFields.NotStarted:
											//http://sharepointserver.yourschool.edu/_layouts/PrepUploadDownload.aspx?fileName=someones_ideas.doc&assignmentName=TheAssignment&ideasAuthorName=someones_user_id&prepAction=upload
											docImage.Append("<A href=\"" + SPEncode.HtmlEncode(PREPWeb.Url + "/_layouts/PrepUploadDownload.aspx?fileName=" + SPEncode.UrlEncode(reviewTask.DocumentName) + "&assignmentName=" + prepAssignment.AssignmentName + "&ideasAuthorName=" + colUserLoginName + "&prepAction=checkout") + "\">");
											imageName = PREPImages.MyTaskNotStarted;
											//altTag = reviewTask.AssignedTo + " to review " + reviewTask.DocumentAuthor + "'s ideas. Status: " + reviewTask.Status + " CLICK HERE TO BEGIN";
											altTag = rowUser.Name + " to review " + colUser.Name + "'s ideas. Status: " + reviewTask.Status + " CLICK HERE TO BEGIN";
											break;
										case PREPFields.InProgress:
											//http://sharepointserver.yourschool.edu/_layouts/PrepUploadDownload.aspx?fileName=someones_ideas.doc&assignmentName=TheAssignment&ideasAuthorName=someones_user_id&prepAction=upload
											docImage.Append("<A href=\"" + SPEncode.HtmlEncode(PREPWeb.Url + "/_layouts/PrepUploadDownload.aspx?assignmentName=" + prepAssignment.AssignmentName + "&ideasAuthorName=" + colUserLoginName + "&prepAction=upload") + "\">");
											imageName = PREPImages.MyTaskInProgress;
											//altTag = reviewTask.AssignedTo + " reviewing " + reviewTask.DocumentAuthor + "'s ideas. Status: " + reviewTask.Status + " CLICK HERE TO UPLOAD A REVIEW";
											altTag = rowUser.Name + " reviewing " + colUser.Name + "'s ideas. Status: " + reviewTask.Status + " CLICK HERE TO UPLOAD A REVIEW";
											break;
										case PREPFields.Completed:
											PREPDocument reviewDoc = PREPDocument.GetReviewDoc(PREPWeb, prepAssignment.AssignmentName, colUserLoginName, rowUserLoginName);
											if(reviewDoc.Exists)
												docImage.Append("<A href=\"" + SPEncode.HtmlEncode(PREPWeb.Url + "/_layouts/PrepUploadDownload.aspx?fileName=" + SPEncode.UrlEncode(reviewDoc.FileName) + "&assignmentName=" + prepAssignment.AssignmentName + "&ideasAuthorName=" + colUserLoginName + "&prepAction=download") + "\">");
											else
												docImage.Append("<A href=\"#\">");
											imageName = PREPImages.TaskCompleted;
											//altTag = reviewTask.AssignedTo + " reviewed " + reviewTask.DocumentAuthor + "'s ideas. Status: " + reviewTask.Status;
											altTag = rowUser.Name + " reviewed " + colUser.Name + "'s ideas. Status: " + reviewTask.Status;
											break;
										default:
											imageName = PREPImages.TaskNotStarted;
											altTag = "Task=" + reviewTask.Title;
											break;
									}

									//display portion of the anchor: an image
									docImage.Append(HTMLTools.Image(imageName, altTag));
									docImage.Append("</a>" + Environment.NewLine);
									prepMatrix.Append(docImage.ToString());
								}
								else
								{
									// for other users, show an icon that is not a hyperlink, unless the task is completed (then it will link to the associated review document)
									switch (statusCode)
									{
										case PREPFields.NotStarted:
											imageName = PREPImages.TaskNotStarted;
											//altTag = reviewTask.AssignedTo + " to review " + reviewTask.DocumentAuthor + "'s ideas. ";
											altTag = rowUser.Name + " to review " + colUser.Name + "'s ideas. ";
											break;
										case PREPFields.InProgress:
											imageName = PREPImages.TaskInProgress;
											//altTag = reviewTask.AssignedTo + " reviewing " + reviewTask.DocumentAuthor + "'s ideas. ";
											altTag = rowUser.Name + " reviewing " + colUser.Name + "'s ideas. ";
											break;
										case PREPFields.Completed:
											PREPDocument reviewDoc = PREPDocument.GetReviewDoc(PREPWeb, prepAssignment.AssignmentName, colUserLoginName, rowUserLoginName);
											if(reviewDoc.Exists)
												docImage.Append("<A href=\"" + SPEncode.HtmlEncode(PREPWeb.Url + "/_layouts/PrepUploadDownload.aspx?fileName=" + SPEncode.UrlEncode(reviewDoc.FileName) + "&assignmentName=" + prepAssignment.AssignmentName + "&ideasAuthorName=" + colUserLoginName + "&prepAction=download") + "\">");
											else
												docImage.Append("<A href=\"#\">");
											imageName = PREPImages.TaskCompleted;
											//altTag = reviewTask.AssignedTo + " reviewed " + reviewTask.DocumentAuthor + "'s ideas. ";
											altTag = rowUser.Name + " reviewed " + colUser.Name + "'s ideas. ";
											break;
										default:
											imageName = PREPImages.TaskNotStarted;
											altTag = "Task=" + reviewTask.Title;
											break;
									}
									
									// add the status to the alt tag
									altTag += "Status: " + reviewTask.Status;
									
									// all users see a link to the review document, if it is completed
									if(statusCode == PREPFields.Completed)
									{
										//display portion of the anchor: an image
										docImage.Append(HTMLTools.Image(imageName, altTag));
										docImage.Append("</a>" + Environment.NewLine);
										prepMatrix.Append(docImage.ToString());
									}
									else
									{
										// if the review document has not yet been uploaded, users
										// not assigned to review it here will see the task for 
										// the user who is, but it will not be a hyperlink.
										prepMatrix.Append(HTMLTools.Image(imageName, altTag));
									}
								}
							} // reviewTask.Exists
						}
							
						// End the User column
						prepMatrix.Append(HTMLTools.Comment("End User column"));
						prepMatrix.Append(HTMLTools.EndCol());
					}
					prepMatrix.Append(HTMLTools.EndRow());
				} // for each user

				prepMatrix.Append(HTMLTools.EndTable());
				prepMatrix.Append(HTMLTools.EmptyLine());
				prepMatrix.Append(HTMLTools.Comment("-=-=-= End PREP Matrix =-=-=-"));
			} // for each assignment

			return prepMatrix.ToString();

		}

	}

}
