/*=====================================================================
  File:      BaseEventSink.cs

  Summary:   Base class for cached event sinks.

---------------------------------------------------------------------
  This file is part of the Windows SharePoint Services SDK Code Samples.

  Copyright (C) Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/
using System;
using System.Xml;
using System.Runtime.InteropServices;
using System.Security.Principal;
using System.IO;
using System.Diagnostics;
using System.Text;
using System.Collections;
using System.Collections.Specialized;
using Microsoft.SharePoint;

namespace PREPWorkflow
{
	/// <summary>
	/// Base class for cached event sinks. 
	/// Handles impersonation and caching event sink classes to handle multiple events on 
	/// the same list. 
	/// This class is part of the Windows SharePoint Services SDK Code Samples. 
	/// Copyright (C) Microsoft Corporation.  All rights reserved.
	/// </summary>
	public abstract class BaseEventSink : Microsoft.SharePoint.IListEventSink
	{
		/// <summary>
		/// Default Constructor
		/// </summary>
		public BaseEventSink()
		{
		}

		/// <summary>
		/// Implementation of the OnEvent method. Windows SharePoint Services will
		/// call this method when an event occurs.
		/// </summary>
		/// <param name="listEvent">The SPListEvent object that describes the event which occured.</param>
		public virtual void OnEvent(Microsoft.SharePoint.SPListEvent listEvent)
		{
			if (listEvent == null)
			{
				throw new ArgumentNullException("listEvent", "list event object cannot be null");
			}
            
			BaseEventSink sink = GetCachedEventSink(listEvent);

			WindowsImpersonationContext wip = null;

			//Ensure each sink instance handles only one event at a time.
			lock(sink)
			{
				try
				{   
					//Make sure the sink class has the current SPListEvent object
					sink.EventInfo = listEvent;

					//Impersonate the appropriate user
					WindowsIdentity id = sink.HandlerIdentity;
					if (id != null)
						wip = id.Impersonate();

					sink.HandleEvent();
                    
				}
				finally
				{ 
					//Cleanup
					if (wip != null)
						wip.Undo();
					if (m_web != null)
						m_web.Close();
                    
					//Null out cached SPWeb and SPList objects so that
					//fresh ones are obtained for the next event.
					m_web = null;
					m_list = null;
				}
			}
		}

		/// <summary>
		/// HandleEvent is called when an event occurs. Child classes should perform the main
		/// event handling actions here.
		/// </summary>
		protected abstract void HandleEvent();
        
		/// <summary>
		/// The SPListEvent object that describes the current event.
		/// </summary>
		protected virtual SPListEvent EventInfo
		{
			get
			{
				return m_ListEvent;
			}
			set
			{
				m_ListEvent = value;
				m_web = null;
				m_list = null;
				m_fileUrl = null;
				m_sinkData = null;
			}
		}
        
		/// <summary>
		/// The SPWeb object for the web site that contains the list
		/// that the event occured on.
		/// </summary>
		protected virtual SPWeb EventWeb
		{
			get
			{
				if (m_web == null)
				{
					m_web = m_ListEvent.Site.OpenWeb();
				}
				return m_web;
			}
		}

		/// <summary>
		/// The SPList object for the list the event occured on.
		/// </summary>
		protected virtual SPList EventList
		{
			get
			{
				if (m_list == null)
				{
					m_list = EventWeb.Lists[EventInfo.ListID];
				}
				return m_list;
			}
		}

		/// <summary>
		/// The url of the file the event occured on.
		/// Equals UrlAfter if UrlAfter is not empty. Otherwise equals
		/// UrlBefore.
		/// </summary>
		protected string EventFileUrl
		{
			get
			{
				if (m_fileUrl == null)
				{
					string webUrl = m_ListEvent.WebUrl == null ? "<NULL>" : m_ListEvent.WebUrl;
					string webRelUrl = m_ListEvent.UrlAfter;
					if (webRelUrl == null || webRelUrl.Length == 0)
					{
						webRelUrl = (m_ListEvent.UrlBefore == null || m_ListEvent.UrlBefore.Length == 0 ?
							"<NULL>" : m_ListEvent.UrlBefore);
					}    
					m_fileUrl = String.Format("{0}/{1}", webUrl, webRelUrl);
				}
				return m_fileUrl;
			}
		}
        
		/// <summary>
		/// Sink Data
		/// </summary>
		protected string Data
		{
			get
			{
				if (m_sinkData == null)
				{
					m_sinkData = m_ListEvent.SinkData;
				}
				return m_sinkData;
			}
		}
		/// <summary>
		/// The WindowsIdentity class of the identity the Event Sink should impersonate.
		/// </summary>
		protected virtual WindowsIdentity HandlerIdentity
		{
			get
			{
				return null;
			}
		}

		/// <summary>
		/// Helper method to handle creating a WindowsIdentity object from a username / domain / password.
		/// </summary>
		/// <param name="User">The username of the account to impersonate.</param>
		/// <param name="Domain">The domain of the account to impersonate.</param>
		/// <param name="Password">The password of the account to impersonate.</param>
		/// <returns></returns>
		public static WindowsIdentity CreateIdentity(string User, string Domain, string Password)
		{
			// The Windows NT user token.
			IntPtr tokenHandle = new IntPtr(0);
        
			const int LOGON32_PROVIDER_DEFAULT = 0;
			const int LOGON32_LOGON_NETWORK_CLEARTEXT = 3;

			tokenHandle = IntPtr.Zero;

			// Call LogonUser to obtain a handle to an access token.
			bool returnValue = LogonUser(User, Domain, Password, 
				LOGON32_LOGON_NETWORK_CLEARTEXT, LOGON32_PROVIDER_DEFAULT,
				ref tokenHandle);
			if (false == returnValue)
			{
				int ret = Marshal.GetLastWin32Error();
				throw new Exception("LogonUser failed with error code: " +  ret);
			}


			//The WindowsIdentity class makes a new copy of the token.
			//It also handles calling CloseHandle for the copy.
			WindowsIdentity id = new WindowsIdentity(tokenHandle);
			CloseHandle(tokenHandle);
			return id;
		}

		/// <summary>
		/// Get the cached event sink object to handle this event.
		/// </summary>
		/// <returns>The cached IListEventSink object that should handle the current event.</returns>
		private BaseEventSink GetCachedEventSink(SPListEvent evt)
		{
			BaseEventSink sink = null;
            
			//Syncrhonize both reads and writes to the cache.
			//Even though the Hashtable class is threadsafe for reads, we don't want to have 
			//multiple sink instances per list. Otherwise, if many events occur on a list 
			//right off the bat, you could get many instances that all have to initialize 
			//themselves.
			lock(SinkCache.SyncRoot)
			{
				sink = SinkCache[evt.ListID] as BaseEventSink;
            
				//The cached sink is only fit if it is the same type and has the same data
				//as the current sink.
				if (sink == null || 
					! sink.GetType().Equals(this.GetType()) ||
					evt.SinkData != sink.Data)
				{
					//Update the cache, throw away the old sink if it exists.
					SinkCache[evt.ListID] = this;
					sink = this;
				}
			}
			return sink;
		}

		/// <summary>
		/// SinkCache
		/// </summary>
		protected static Hashtable SinkCache = new Hashtable();

		[DllImport("advapi32.dll", SetLastError=true)]
		private static extern bool LogonUser(String lpszUsername, String lpszDomain, String lpszPassword, 
			int dwLogonType, int dwLogonProvider, ref IntPtr phToken);

		[DllImport("kernel32.dll", CharSet=CharSet.Auto)]
		private extern static bool CloseHandle(IntPtr handle);

		/// <summary>
		/// Cached List Event
		/// </summary>
		protected SPListEvent m_ListEvent;
		private SPWeb m_web;
		private SPList m_list;
		private string m_fileUrl;
		private string m_sinkData;
        
	}
}
