/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using System.Collections;
using System.Configuration;
using System.Text;

namespace PREPWorkflow
{
	/// <summary>
	/// Class which represents a PREP Assignment. A PREP Assignment is persisted in SharePoint 
	/// as three Events on the Events List: Assignment Given, Ideas Due, and Reviews Due. 
	/// </summary>
	public class PREPAssignment
	{
		private string _AssignmentName;
		private string _Description;
		private bool _HasData;
		private DateTime _AssignedDate;
		private DateTime _IdeasDueDate;
		private DateTime _ReviewsDueDate;

		/// <summary>
		/// Default constructor
		/// </summary>
		public PREPAssignment()
		{}

		/// <summary>
		/// Constructor which populates a PREPAssignment object by doing a lookup on a persisted
		/// assignment using the assignment name. 
		/// </summary>
		/// <param name="PREPWeb">The SharePoint Web object</param>
		/// <param name="PREPAssignment_Key">The key is the assignment name</param>
		public PREPAssignment(SPWeb PREPWeb, string PREPAssignment_Key)
		{
			//Obtain a collection of objects representing all PREP assignments, with their titles and dates
			ArrayList prepAssignments = PREPAssignment.PREPAssignment_Collection(PREPWeb);
			
			// find the PREPAssignment object associated with the passed-in assignmentName
			foreach(PREPAssignment prepAssignment in prepAssignments)
			{
				if(prepAssignment.AssignmentName == PREPAssignment_Key)
				{
					this._AssignmentName = prepAssignment.AssignmentName;
					this._Description = prepAssignment.Description;
					this._HasData = prepAssignment.HasData;
					this._AssignedDate = prepAssignment.AssignedDate;
					this._IdeasDueDate = prepAssignment.IdeasDueDate;
					this._ReviewsDueDate = prepAssignment.ReviewsDueDate;
					break;
				}
			}
		}

	
		/// <summary>
		/// Returns an ArrayList of PREPAssignment objects representing all of the valid PREP Assignments for this site. 
		/// These objects are created by reading the Events list, looking for PREP Assignment events.
		/// PREP Assignment events are identified by having a value in the Assignment Name field.
		/// </summary>
		/// <param name="web">The current SharePoint web</param>
		/// <returns></returns>
		public static ArrayList PREPAssignment_Collection(SPWeb web)
		{
			ArrayList result = new ArrayList();
			string prepDocLibraryName = ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"];
			string prepTaskListName = ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"];

			try
			{
				SPList eventList = web.Lists["Events"];
				SPListItemCollection eventListCollection = eventList.Items;
				string assignmentName;
				string dueAssignmentName;

				foreach (SPListItem item in eventListCollection)
				{
					if((string)item["Milestone"] == "Assignment Given")
					{
						PREPAssignment CollItem = new PREPAssignment();
						assignmentName = (string)item["Assignment Name"];
						CollItem.AssignmentName = assignmentName;
						CollItem.Description = (string)item["Description"];
						CollItem.AssignedDate = (DateTime)item["EventDate"];
						// find the "Ideas Due" Date
						foreach (SPListItem dueItem in eventListCollection)
						{
							if((string)dueItem["Milestone"] == "Ideas Due")
							{
								dueAssignmentName = (string)dueItem["Assignment Name"];
								if (dueAssignmentName == assignmentName)
								{
									CollItem.IdeasDueDate = (DateTime)dueItem["EventDate"];
									break;
								}
							}
						}
						// find the "Reviews Due" Date
						foreach (SPListItem dueItem in eventListCollection)
						{
							if((string)dueItem["Milestone"] == "Reviews Due")
							{
								dueAssignmentName = (string)dueItem["Assignment Name"];
								if (dueAssignmentName == assignmentName)
								{
									CollItem.ReviewsDueDate = (DateTime)dueItem["EventDate"];
									break;
								}
							}
						}

						// find out if this assignment has any data
						CollItem.HasData = false;
						
						// check tasks
						ArrayList prepTasks = PREPTask.PREPTask_Collection(web.Lists[prepTaskListName]);
						foreach(PREPTask task in prepTasks)
						{
							if(task.AssignmentName == assignmentName)
							{
								CollItem.HasData = true;
								break;
							}
						}

						// check documents
						if(!CollItem.HasData)
						{
							SPFolder docLibFolder = web.Folders[prepDocLibraryName.Replace(" ","")];
							SPFileCollection docLibFiles = docLibFolder.Files;
							foreach(SPFile file in docLibFiles)
							{
								if(file.Name.ToUpper().IndexOf(assignmentName.ToUpper()) > -1)
								{
									CollItem.HasData = true;
								}
							}

						}

						//Add the item to the collection
						result.Add(CollItem);
					}
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in PREPAssignment_Collection().", ex);
			}

			return result;

		}

		/// <summary>
		/// Creates a PREP Assignment and returns the number of Events that were created. 
		/// A PREP Assignment consists of three Events on the Events List: Assignment Given, 
		/// Ideas Due, and Reviews Due.
		/// </summary>
		/// <returns>The number of Events that were created.</returns>
		public static int Create(SPWeb web, PREPAssignment assignment)
		{
			int numEvents = 0;
			ArrayList prepAssignments;
			try
			{
				SPList eventList = web.Lists["Events"];

				prepAssignments = PREPAssignment.PREPAssignment_Collection(web);
				
				// check that there is not a pre-existing assignment with the same name
				foreach(PREPAssignment prepAssignment in prepAssignments)
				{
					if(prepAssignment.AssignmentName == assignment.AssignmentName)
					{
						throw new Exception("PREP Assignment " + assignment.AssignmentName + " already exists. Please choose another name.");
					}
				}

				SPListItemCollection items= eventList.Items;

				// Create the Assignment Given Event
				SPListItem item = items.Add();
				item["Title"] = assignment.AssignmentName + " - Assignment Given";
				item["Description"] = assignment.Description;
				item["Begin"] = assignment.AssignedDate;
				item["Milestone"] = "Assignment Given";
				item["Assignment Name"] = assignment.AssignmentName;
				// Commit the changes
				item.Update();
				numEvents ++;

				// Create the Ideas Due Event
				item = items.Add();
				item["Title"] = assignment.AssignmentName + " - Ideas Due";
				item["Description"] = assignment.Description;
				item["Begin"] = assignment.IdeasDueDate;
				item["Milestone"] = "Ideas Due";
				item["Assignment Name"] = assignment.AssignmentName;
				// Commit the changes
				item.Update();
				numEvents ++;

				// Create the Reviews Due Event
				item = items.Add();
				item["Title"] = assignment.AssignmentName + " - Reviews Due";
				item["Description"] = assignment.Description;
				item["Begin"] = assignment.ReviewsDueDate;
				item["Milestone"] = "Reviews Due";
				item["Assignment Name"] = assignment.AssignmentName;
				// Commit the changes
				item.Update();
				numEvents ++;

			}
			catch (Exception ex)
			{
				Utilities.LogError(web, "Error in PREPAssignment.Create().", ex);
				throw ex;
			}

			return numEvents;
		}
		
		/// <summary>
		/// Deletes a PREP Assignment and returns the number of Events that were deleted. 
		/// A PREP Assignment consists of three Events on the Events List: Assignment Given, 
		/// Ideas Due, and Reviews Due.
		/// </summary>
		/// <returns>The number of Events that were deleted.</returns>
		public static int Delete(SPWeb web, string assignmentName)
		{
			ArrayList prepAssignments;
			int numEvents = 0;
			try
			{
				
				SPList eventList = web.Lists["Events"];

				prepAssignments = PREPAssignment.PREPAssignment_Collection(web);
				
				// check to see if there is existing data for this assignment. If so, will not delete.
				foreach(PREPAssignment prepAssignment in prepAssignments)
				{
					if(prepAssignment.AssignmentName == assignmentName)
					{
						//if this assignment has existing data (tasks or documents), do not allow deletion
						if(prepAssignment.HasData)
						{
							throw new Exception("PREP Assignment " + assignmentName + " has tasks or documents. It cannot be deleted.");
						}
					}
				}
				
				SPListItemCollection eventListCollection = eventList.Items;

				// Loop through the collection backwards, so that deleting items 
				// does not throw off the index.
				for(int i=eventListCollection.Count-1; i>-1; i--)
				{
					if((string)eventListCollection[i]["Assignment Name"] == assignmentName)
					{
						eventListCollection.Delete(i);
						numEvents ++;
					}
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError(web, "Error in PREPAssignment.Delete().", ex);
				throw ex;
			}

			return numEvents;
		}

		/// <summary>
		/// Loops through the PREP Tasks associated with an assignment, and returns true 
		/// if all tasks are completed. 
		/// </summary>
		/// <returns>true if the assignment is complete for all users.</returns>
		public static bool IsComplete(SPWeb PREPWeb, string assignmentName)
		{
			try
			{
				SPList prepTaskList = PREPWeb.Lists[ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"]];

				ArrayList prepTasks = PREPTask.PREPTask_Collection(prepTaskList, assignmentName);
				// return false if no tasks are in the collection
				if(prepTasks.Count == 0)
					return false; 
				//Every single task in the assignment has to be completed for the assignment to be completed
				foreach (PREPTask task in prepTasks)
				{
					if (task.Status != PREPFields.Completed)
						return false;
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in PREPAssignment.IsComplete()", ex);
				return false;
			}
			// if we made it to here, every task is completed.
			return true;
		}

		/// <summary>
		/// Dumps the title and status of all tasks related to an assignment. 
		/// </summary>
		/// <param name="PREPWeb">A SharePoint SPWeb (site) object set to the current site context.</param>
		/// <param name="assignmentName">The PREP Assignment name.</param>
		/// <param name="newLine">The newline character. Use a br tag for html pages, Environment.Newline for emails.</param>
		/// <returns></returns>
		public static string StatusReport(SPWeb PREPWeb, string assignmentName, string newLine)
		{
			string report = "";
			StringBuilder rpt = new StringBuilder();
			try
			{
				SPList prepTaskList = PREPWeb.Lists[ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"]];

				ArrayList prepTasks = PREPTask.PREPTask_Collection(prepTaskList, assignmentName);

				// return if no tasks are in the collection
				if(prepTasks.Count == 0)
					return report;
 
				rpt.Append("Site: " + PREPWeb.Title + Environment.NewLine);
				rpt.Append("URL:  " + PREPWeb.Url + Environment.NewLine + Environment.NewLine);
				
				rpt.Append("Status of Assignment " + assignmentName + ":" + newLine + newLine);
				//loop through the task collection
				foreach (PREPTask task in prepTasks)
				{
					rpt.Append("Task: " + task.Title + " Status: " + task.Status + newLine + newLine);
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in PREPAssignment.StatusReport()", ex);
				rpt.Append(newLine + "Report not complete. Exiting with errors.");
				return rpt.ToString();
			}
			
			return rpt.ToString();
		}			

		/// <summary>
		/// The PREP Assignment name
		/// </summary>
		public string AssignmentName{get{return _AssignmentName;}set{_AssignmentName = value;}}

		/// <summary>
		/// PREP Assignment description.
		/// </summary>
		public string Description{get{return _Description;}set{_Description = value;}}

		/// <summary>
		/// Returns true if there are any tasks or documents associated with this PREP Assignment.
		/// </summary>
		public bool HasData{get{return _HasData;}set{_HasData = value;}}

		/// <summary>
		/// Date this PREP Assignment begins.
		/// </summary>
		public DateTime AssignedDate{get{return _AssignedDate;}set{_AssignedDate = value;}}

		/// <summary>
		/// Date on which an Ideas document is due for this PREP Assignment.
		/// </summary>
		public DateTime IdeasDueDate{get{return _IdeasDueDate;}set{_IdeasDueDate = value;}}

		/// <summary>
		/// Date on which the Review documents are due for this PREP Assignment.
		/// </summary>
		public DateTime ReviewsDueDate{get{return _ReviewsDueDate;}set{_ReviewsDueDate = value;}}
	}

}
