/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using System.Collections;
using System.Configuration;


namespace PREPWorkflow
{
	/// <summary>
	/// Class which represents a PREP Document.
	/// </summary>
	public class PREPDocument
	{
		private string _FileName;
		private string _AssignmentName;
		private string _EventUserLogin;
		private string _EventUserName;
		private string _IdeasAuthorLogin;
		private string _Folder;
		private SPFile _SPFileObject;
		private bool _PREP;
		private string _DocType;
		private bool _Exists;

		/// <summary>
		/// Structure which contains information about a PREP Document.
		/// </summary>
		public struct DocInfo
		{
			/// <summary>
			/// The PREP Document filename and extension.
			/// </summary>
			public string FileName;

			/// <summary>
			/// The PREP Assignment name.
			/// </summary>
			public string AssignmentName;

			/// <summary>
			/// The author of the Ideas Document the current document pertains to. This is equal to the 
			/// author of this document if this is an Ideas document, but not if this is a Review document.
			/// </summary>
			public string IdeasAuthorLoginName;

			/// <summary>
			/// Returns true if this DocInfo structure contains information about a valid PREP Document.
			/// </summary>
			public bool IsValid;
		}

		/// <summary>
		/// Default Constructor
		/// </summary>
		public PREPDocument()
		{}

		/// <summary>
		/// Constructor which populates a PREPDocument object from an SPFile object
		/// </summary>
		/// <param name="thisFile">A SharePoint SPFile object representing the PREP file to be constructed.</param>
		public PREPDocument(SPFile thisFile)
		{
			try
			{
				// Get the file as an SPListItem object.
				SPListItem thisFileAsListItem = thisFile.Item;

				// Get the assignment name and the user whose ideas the document pertains to from the 
				// folder the document resides in.
				// Folder structure is assumed to be:
				// PREPDocuments/userID/AssignmentName
				// The User ID is the person who wrote the ideas doc, not necessarily the person who created the current file.
				DocInfo docInfo = PREPDocument.GetDocInfoFromURL(thisFile.Url);
				// If the folder structure isn't correct, exit.
				if(!docInfo.IsValid)
				{
					this.Exists = false;
					return;
				}
				this._IdeasAuthorLogin = docInfo.IdeasAuthorLoginName;
				this._AssignmentName = docInfo.AssignmentName;

				// get the boolean which determines whether or not this is a PREP document.
				// We have to get this from the ListItem view of the document, not the SPFile view.
				// There is one PREP document per user per folder. 
				// If the document author == the user folder, and PREP is true, it is an ideas document.
				// If the document author != the user folder, and PREP is true, it is a review.
				// If PREP is false, it is an extra document uploaded to the folder.
				try{this._PREP = (bool)thisFileAsListItem["PREP"];}
				catch{this._PREP = false;}

				// for clarity, a field that makes the document type explicit has been added.
				try{this._DocType = thisFileAsListItem["DocType"].ToString();}
				catch{this._DocType="";}

				this._FileName = thisFile.Name;
				this._Folder = thisFile.ParentFolder.Url; //thisFile.Url;

				this._SPFileObject = thisFile;

				this._Exists = true;
			}
			catch (Exception ex)
			{
				Utilities.LogError(thisFile.ParentFolder.ParentWeb, "Error in PREPDocument(SPFile thisFile)", ex);
			}
		}

		/// <summary>
		/// Constructor which populates an instance of the PREPDocument object with data from
		/// an event in the document library
		/// </summary>
		/// <param name="EventWeb">The SharePoint Web object on which the Event occurred.</param>
		/// <param name="theEvent">The Event object. Contains information about the Event, such as which document was affected and which user triggered it.</param>
		/// <returns></returns>
		public PREPDocument(SPWeb EventWeb, SPListEvent theEvent)
		{

			try
			{
				SPFolder docLibFolder = EventWeb.GetFolder(theEvent.UrlAfter.Substring(0, theEvent.UrlAfter.LastIndexOf("/")));
				SPFileCollection docLibFiles = docLibFolder.Files;
				// Get the file as an SPFile object.
				SPFile thisFile = docLibFiles[theEvent.UrlAfter];
				// Get the file as an SPListItem object.
				SPListItem thisFileAsListItem = thisFile.Item;

				// Get the assignment name and the user whose ideas the document pertains to from the 
				// folder the document resides in.
				// Folder structure is assumed to be:
				// PREPDocuments/userID/AssignmentName
				// The User ID is the person who wrote the ideas doc, not necessarily the person who created the current file.
				DocInfo docInfo = PREPDocument.GetDocInfoFromURL(thisFile.Url);
				// If the folder structure isn't correct, exit.
				if(!docInfo.IsValid)
				{
					this.Exists = false;
					return;
				}
				this._IdeasAuthorLogin = docInfo.IdeasAuthorLoginName;
				this._AssignmentName = docInfo.AssignmentName;

				// get the boolean which determines whether or not this is a PREP document.
				// We have to get this from the ListItem view of the document, not the SPFile view.
				// There is one PREP document per user per folder. 
				// If the document author == the user folder, and PREP is true, it is an ideas document.
				// If the document author != the user folder, and PREP is true, it is a review.
				// If PREP is false, it is an extra document uploaded to the folder.
				try{this._PREP = (bool)thisFileAsListItem["PREP"];}
				catch{this._PREP = false;}

				// for clarity, a field that makes the document type explicit has been added.
				try{this._DocType = thisFileAsListItem["DocType"].ToString();}
				catch{this._DocType = "";}


				//thisDocument.FileName = Utilities.GetFileName(theEvent.UrlAfter);
				this._FileName = thisFile.Name;
				this._Folder = thisFile.ParentFolder.Url; //thisFile.Url;
				this._EventUserLogin = Utilities.GetLoginName(theEvent.UserLoginName);
				if(theEvent.UserDisplayName != "")
				{
					this._EventUserName = theEvent.UserDisplayName;
				}
				else
				{
					this._EventUserName = this._EventUserLogin;
				}

				this._SPFileObject = thisFile;

				this._Exists = true;
			}
			catch (Exception ex)
			{
				this._Exists = false;
				Utilities.LogError(EventWeb, "Error in PREPDocument(SPWeb EventWeb, SPListEvent theEvent)", ex);
			}
		}

			
		/// <summary>
		/// Copies the current document to a new name based on a passed-in identifier. 
		/// New file name will be in the form "originalfileName_identifier_random8CharString.ext"
		/// e.g. "Asbestos Canoe.doc" becomes "Asbestos Canoe_mike_32CCD49F.doc". 
		/// This method was intended to rename and copy files to avoid unintentional overwrites, in the 
		/// Update event in the Event Handler. Currently it is not in use.
		/// </summary>
		/// <param name="identifier">A string to be appended to the copied document file name.</param>
		/// <returns>The new file URL</returns>
		public string CopyToNewName(string identifier)
		{
			string newURL = "";
			try
			{
				string folderPath = this.SPFileObject.ParentFolder.Url;
				string fileRoot = this.FileName.Substring(0, this.FileName.LastIndexOf("."));
				string fileExt = this.FileName.Substring(fileRoot.Length +1, this.FileName.Length - fileRoot.Length - 1);
				newURL = folderPath + "/" + fileRoot + "_" + identifier + "_" + Utilities.GetRandomString(8) + "." + fileExt;
				this.SPFileObject.CopyTo(newURL);
			}
			catch(Exception ex)
			{
				Utilities.LogError(this.SPFileObject.Item.ParentList.ParentWeb, "Error in Utilities.CopyToNewName(). identifier= " + identifier, ex);
			}
			return newURL;
		}

		/// <summary>
		/// Returns an ArrayList of all PREPDocument objects in a folder.
		/// </summary>
		/// <param name="folder">The folder containing the PREP Documents to return</param>
		/// <returns></returns>
		public static ArrayList PREPDocument_Collection(SPFolder folder)
		{
			ArrayList prepDocuments = new ArrayList();
			try
			{
				if(folder.Exists)
				{
					SPFileCollection docLibFiles = folder.Files;
					
					foreach(SPFile file in docLibFiles)
					{
						PREPDocument thisDocument = new PREPDocument(file);

						prepDocuments.Add(thisDocument);
					}
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError(folder.ParentWeb, "Error in PREPDocument_Collection(SPFolder folder) " , ex);
			}

			return prepDocuments;

		}

		/// <summary>
		/// Returns the PREP Document object representing the Ideas Document for a given user (passed in as userLoginName)
		/// for a given assignment (assignmentName).
		/// If no such document can be found, Boolean field Exists is set to false.
		/// </summary>
		/// <param name="PREPWeb">The current SharePoint site</param>
		/// <param name="assignmentName">PREP Assignment name</param>
		/// <param name="userLoginName">The User Login name of the user whose PREP ideas document is to be retrieved.</param>
		/// <returns>PREPDocument object containing the user's Ideas Document.</returns>
		public static PREPDocument GetIdeasDoc(SPWeb PREPWeb, string assignmentName, string userLoginName)
		{
			PREPDocument resultDoc = new PREPDocument();
			resultDoc.Exists = false;
			try
			{
				string prepDocLibraryName = ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"].Replace(" ", "");
				SPFolder assignmentFolder;
				assignmentFolder = PREPWeb.GetFolder(prepDocLibraryName + "/" + userLoginName + "/" + assignmentName);
				if(assignmentFolder.Exists)
				{
					ArrayList docList = PREPDocument.PREPDocument_Collection(assignmentFolder);
					foreach(PREPDocument doc in docList)
					{
						// Check 3 things:
						// 1. Author of the ideas document that the PREP document pertains to is the same as the currently logged-in user.
						//    Note that this could be either an Ideas document or a Review document; hence check #3 below.
						// 2. document is a PREP document
						// 3. Author of the PREP document is the currently logged-in user. 
						if(doc.IdeasAuthorLogin == userLoginName && doc.PREP == true && Utilities.GetLoginName(doc.SPFileObject.Author.LoginName) == userLoginName)
						{
							resultDoc = doc;
							resultDoc.Exists = true;
							return resultDoc;
						}
					}
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError(PREPWeb, "Error in PREPDocument.GetIdeasDoc() " , ex);
			}

			return resultDoc;
		}

		/// <summary>
		/// Returns the PREP Document object representing the Review Document for a given reviewer (reviewAuthorLoginName),
		/// reviewing the ideas of a given author (ideasAuthorLoginName), for a given assignment (assignmentName).
		/// If no such document can be found, Boolean field Exists is set to false.
		/// </summary>
		/// <param name="PREPWeb">The current SharePoint site</param>
		/// <param name="assignmentName">PREP Assignment name</param>
		/// <param name="ideasAuthorLoginName">Login Name of the author of the Ideas Document which is the subject of this Review document</param>
		/// <param name="reviewAuthorLoginName">Login Name of the author of this Review Document</param>
		/// <returns>A PREP Review Document</returns>
		public static PREPDocument GetReviewDoc(SPWeb PREPWeb, string assignmentName, string ideasAuthorLoginName, string reviewAuthorLoginName)
		{
			PREPDocument resultDoc = new PREPDocument();
			resultDoc.Exists = false;
			try
			{
				// A review task cannot be assigned to the same user who wrote the ideas document
				if(ideasAuthorLoginName != reviewAuthorLoginName)
				{
					string prepDocLibraryName = ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"].Replace(" ", "");
					SPFolder assignmentFolder;
					assignmentFolder = PREPWeb.GetFolder(prepDocLibraryName + "/" + ideasAuthorLoginName + "/" + assignmentName);
					if(assignmentFolder.Exists)
					{
						ArrayList docList = PREPDocument.PREPDocument_Collection(assignmentFolder);
						foreach(PREPDocument doc in docList)
						{
							// if it's a PREP document pertaining to the ideas document author in question
							if(doc.IdeasAuthorLogin == ideasAuthorLoginName && doc.PREP == true)
							{
								//get the author of this document
								string thisDocAuthorLoginName = Utilities.GetLoginName(doc.SPFileObject.Author.LoginName);
								if(thisDocAuthorLoginName == reviewAuthorLoginName)
								{
									resultDoc = doc;
									resultDoc.Exists = true;
									return resultDoc;
								}
							}
						}
					}
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError(PREPWeb, "Error in PREPDocument.GetReviewDoc() " , ex);
			}

			return resultDoc;
		}

		/// <summary>
		/// Takes a URL and returns a DocInfo object. This object breaks the URL out into 
		/// the parts of the DocInfo object: IdeasAuthorLoginName, AssignmentName, and FileName. 
		/// Additionally, there is a boolean IsValid which is set to false if the URL is not in 
		/// the proper format. It does not check the validity of any of the data (i.e. it doesn't know 
		/// if the user does not exist or the assignment name does not exist).
		/// </summary>
		/// <param name="url">A SharePoint Document URL</param>
		/// <returns>A DocInfo object containing IdeasAuthorLoginName, AssignmentName, and FileName.</returns>
		public static DocInfo GetDocInfoFromURL(string url)
		{
			DocInfo docInfo = new DocInfo();
			char[] slash = {'/'};
			string[] urlParts = url.Split(slash);
			if(urlParts.Length != 4)
			{
				docInfo.IsValid = false;
			}
			else
			{
				docInfo.IdeasAuthorLoginName = urlParts[1];
				docInfo.AssignmentName = urlParts[2];
				docInfo.FileName = urlParts[3];
				docInfo.IsValid = true;
			}
			return docInfo;
		}

		/// <summary>
		/// Returns true if the author of the passed-in file does not already
		/// have a PREP file in the current folder. Used in the Event Handler 
		/// to make sure that each user has only one PREP document in a given folder. 
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="prepDoc">A PREP Document object</param>
		/// <returns>Returns true if the author of the passed-in file does not already 
		/// have a PREP file in the current folder.</returns>
		public static bool IsNewPREPDocument(SPWeb PREPWeb, PREPDocument prepDoc)
		{
			try
			{
				// See if a file with PREP = true already exists for the current user in the folder the
				// current file has just been placed into.
				SPFolder docLibFolder = PREPWeb.GetFolder(prepDoc.Folder);
				SPFileCollection docLibFiles = docLibFolder.Files;
				foreach(SPFile file in docLibFiles)
				{
					// ignore the just-uploaded file (which is already in the folder)
					if(file.Name != prepDoc.FileName)
					{
						// if we found a file elsewhere in the list with the same user login as the just-uploaded file
						if (Utilities.GetLoginName(file.Author.LoginName) == prepDoc.EventUserLogin)
						{
							// if the PREP bit is on
							try // the PREP bit may not exist, so it has its own try-catch block just in case.
							{
								if (((bool)file.Item["PREP"] == true))
								{
									// this user already has a PREP file.
									return false;
								}
							}
							catch{}
						}
					}
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError(PREPWeb, "Error in IsNewPREPDocument", ex);
				return false;
			}
			return true;
		}

		/// <summary>
		/// Validates the following: 
		/// 1) The folder structure is in the correct format (PREP Documents/userName/assignmentName) 
		/// 2) The assignment exists 
		/// 3) The userName exists 
		/// 4) The userName belongs to the assignment 
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="folderName">The folder to be checked.</param>
		/// <returns>true if it is a valid PREP folder.</returns>
		public static bool IsPREPFolder (SPWeb PREPWeb, string folderName)
		{
			try
			{
				DocInfo docInfo = new DocInfo();
				char[] slash = {'/'};
				string[] folderParts = folderName.Split(slash);
				// Looking for this format: PREP Documents/userName/assignmentName
				if(folderParts.Length != 3)
				{
					return false;
				}
				else
				{
					docInfo.IdeasAuthorLoginName = folderParts[1];
					docInfo.AssignmentName = folderParts[2];

					// Validate that IdeasAuthorLoginName is a real user name, and AssignmentName is a real assignment name,
					// and that the user belongs in this assignment.

					// If it is a valid assignment, there should be a role with the assignment name.
					// If the user is valid, they will exist within the role. Therefore, to validate
					// everything, we just have to find the user within the role.
				
					PREPRole assignmentRole = new PREPRole(PREPWeb, docInfo.AssignmentName);
					if(assignmentRole.Exists)
					{
						ArrayList assignmentUsers = assignmentRole.Users;
						foreach (SPUserInfo user in assignmentUsers)
						{
							if(Utilities.GetLoginName(user.LoginName) == docInfo.IdeasAuthorLoginName)
								return true;
						}
					}
				}
			}
			catch(Exception ex)
			{
				Utilities.LogError(PREPWeb, "Error in PREPDocument.IsPREPFolder", ex);
			}

				// if we fall through to here, the assignment name or user name is not valid.
				return false;

		}


		/// <summary>
		/// The file name of the PREP document that triggered the event
		/// </summary>
		public string FileName{get{return _FileName;}set{_FileName = value;}}

		/// <summary>
		/// The PREP Assignment name
		/// </summary>
		public string AssignmentName{get{return _AssignmentName;}set{_AssignmentName = value;}}

		/// <summary>
		/// Login Name of the user whose action triggered the event on the document library
		/// </summary>
		public string EventUserLogin{get{return _EventUserLogin;}set{_EventUserLogin = value;}}

		/// <summary>
		/// Display Name of the user whose action triggered the event on the document library
		/// </summary>
		public string EventUserName{get{return _EventUserName;}set{_EventUserName = value;}}

		/// <summary>
		/// Login Name of the author of the ideas document that this document pertains to
		/// </summary>
		public string IdeasAuthorLogin{get{return _IdeasAuthorLogin;}set{_IdeasAuthorLogin = value;}}

		/// <summary>
		/// The folder where the document resides (e.g. PREPDocuments/UserID/AssignmentName)
		/// </summary>
		public string Folder{get{return _Folder;}set{_Folder = value;}}
	
		/// <summary>
		/// The complete SPFile object
		/// </summary>
		public SPFile SPFileObject{get{return _SPFileObject;}set{_SPFileObject = value;}}

		/// <summary>
		/// PREP Bit. If True, this is a PREP Document. 
		/// if PREP == true AND the document author userID == the folder name, it is an ideas document. 
		/// if PREP == true AND the document author userID != the folder name, it is a review.
		/// if PREP == false, it is an extra document uploaded to the folder. 
		/// </summary>
		public bool PREP{get{return _PREP;}set{_PREP = value;}}

		/// <summary>
		/// The PREP Document Type. May be one of two values: Ideas or Review.
		/// </summary>
		public string DocType{get{return _DocType;}set{_DocType = value;}}

		/// <summary>
		/// True if a document exists for a given user and assignment.
		/// </summary>
		public bool Exists{get{return _Exists;}set{_Exists = value;}}
	
	}
	
}
