/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using System.Security.Principal;

namespace PREPWorkflow
{
	/// <summary>
	/// Allows code to impersonate a Windows user that is different from the logged-in user.
	/// Typically this is used to execute code that requires a higher level of privilege than
	/// the user possesses (e.g. something that requires an Administrator).
	/// </summary>
	public class PREPImpersonation
	{
		private WindowsIdentity id = null;
		private WindowsImpersonationContext context = null;

		/// <summary>
		/// Default constructor.
		/// </summary>
		public PREPImpersonation()
		{}

		/// <summary>
		/// Instantiates the object with a set of credentials which will be used for Windows Impersonation.
		/// </summary>
		/// <param name="user"></param>
		/// <param name="domain"></param>
		/// <param name="password"></param>
		public PREPImpersonation(string user, string domain, string password)
		{
			// The heavy lifting is done in the base class of PREPEventSink.
			// This contains the Win32 COM interop code that actually performs the impersonation.
			try
			{
				id = PREPEventSink.CreateIdentity(user, domain, password);
			}
			catch(Exception ex)
			{
				Utilities.LogError("Impersonation Error. ", ex);
			}
		}

		/// <summary>
		/// Begins impersonating the user specified in the constructor.
		/// </summary>
		public void Impersonate()
		{
			try
			{
				this.context = this.id.Impersonate();
			}
			catch(Exception ex)
			{
				Utilities.LogError("Impersonation Error. ", ex);
			}
		}

		/// <summary>
		/// Stops impersonating the user specified in the constructor.
		/// </summary>
		public void UnImpersonate()
		{
			try
			{
				this.context.Undo();
			}
			catch(Exception ex)
			{
				Utilities.LogError("Impersonation Error. ", ex);
			}
		}
	}
}
