/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using System.Collections;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Configuration;

namespace PREPWorkflow
{
	/// <summary>
	/// Custom SPRole object for PREP. 
	/// Each PREP Assignment has a custom role, with the same name, which contains the 
	/// set of users who are included in the assignment. 
	/// This class is a workaround to address the problem where the 
	/// SharePoint Object Model only allows Administrator access to the SPRole object. 
	/// This problem cannot be addressed through impersonation because SharePoint only 
	/// considers the original user context when evaluating security for SPRole. 
	/// This class is intended only for non-administrators to be able to read the user
	/// collections in the role object (e.g., in the PREP Matrix). Therefore, there are no add or update methods.
	/// These will be done using the normal SPRole object by an administrator.
	/// </summary>
	public class PREPRole
	{
		private int _ID;
		private string _Name;
		private string _Description;
		private ArrayList _Users;
		private bool _Exists;

		/// <summary>
		/// Default constructor.
		/// </summary>
		public PREPRole()
		{}

		/// <summary>
		/// Populates a PREPRole object, given the web and role name
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="PREPRole_Key">The name of the PREP Role. These names are equal to assignment names; each assignment has a corresponding role 
		/// which contains the set of users who belong to the assignment.</param>
		public PREPRole(SPWeb PREPWeb, string PREPRole_Key)
		{
			SqlGuid siteID = PREPWeb.Site.ID;
			SqlGuid webID = PREPWeb.ID;

			this._Exists = false;
			
			string privilegedUserID = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Username"]);
			string domain = ConfigurationSettings.AppSettings["PREPWorkflow.Domain"];
			string password = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Password"]);
			try
			{
				// Impersonate a user with sufficient authorization to make SQL calls
				PREPImpersonation imp = null;
				imp = new PREPImpersonation(privilegedUserID, domain, password);
				imp.Impersonate();
				string connString = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Connection"]);

				// Get the ID, Name, and Description from SharePoint for the SPRole designated by the passed-in role name (PREPRole_Key)
				SqlConnection sqlConnection = new SqlConnection(connString);
				SqlCommand sqlCommand = new SqlCommand("proc_SecGetWebGroupByTitle", sqlConnection);
				sqlCommand.CommandType = CommandType.StoredProcedure;
				sqlCommand.Parameters.Add("@SiteId", siteID);
				sqlCommand.Parameters.Add("@WebId", webID);
				sqlCommand.Parameters.Add("@Title", PREPRole_Key);
				sqlConnection.Open();

				SqlDataReader r = sqlCommand.ExecuteReader(CommandBehavior.CloseConnection);
				// Exit if no role was found with the given name
				if(!r.HasRows)
				{
					r.Close();
					this._Exists = false;
					return;
				}
				while (r.Read())
				{
					if(!r.IsDBNull(0)){this._ID = r.GetInt32(0);}
					if(!r.IsDBNull(1)){this._Name = r.GetString(1);}
					if(!r.IsDBNull(2)){this._Description = r.GetString(2);}
					this._Exists = true;
				}
				r.Close();

				//Get the User Collection for the SPRole using the ID retrieved above.
				sqlConnection = new SqlConnection(connString);
				sqlCommand = new SqlCommand("proc_SecListWebGroupMembership", sqlConnection);
				sqlCommand.CommandType = CommandType.StoredProcedure;
				sqlCommand.Parameters.Add("@SiteId", siteID);
				sqlCommand.Parameters.Add("@WebId", webID);
				sqlCommand.Parameters.Add("@GroupId", this._ID);
				sqlConnection.Open();

				r = sqlCommand.ExecuteReader(CommandBehavior.CloseConnection);
				ArrayList userArray = new ArrayList();
				while (r.Read())
				{
					// We can't create SPUser objects from nothing, so we'll use SPUserInfo structs.
					SPUserInfo CollItem = new SPUserInfo();
					// index 0 is the db identity field, which we don't need
					if(!r.IsDBNull(2)){CollItem.Name = r.GetString(2);}
					if(!r.IsDBNull(3)){CollItem.LoginName = r.GetString(3);}
					if(!r.IsDBNull(4)){CollItem.Email = r.GetString(4);}
					if(!r.IsDBNull(5)){CollItem.Notes = r.GetString(5);}
					userArray.Add(CollItem);
				}
				r.Close();

				this._Users = userArray;

				// Stop impersonating
				imp.UnImpersonate();
			}
			catch(Exception ex)
			{
				Utilities.LogError(PREPWeb, "Error in PREPRole(SPWeb PREPWeb, string PREPRole_Key)", ex);				
			}
		}

		/// <summary>
		/// Sends an email message to each member of a custom Role.
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="roleName">The role name. This will correspond to an Assignment Name.</param>
		/// <param name="subject">Subject of the email to send.</param>
		/// <param name="message">Body of the email to send.</param>
		public static void SendMailToMembers(SPWeb PREPWeb, string roleName, string subject, string message)
		{
			PREPRole role = new PREPRole(PREPWeb, roleName);
			foreach(SPUserInfo roleMember in role.Users)
			{
				Utilities.SendEmail(subject, message, roleMember.Email);  
			}

		}

		/// <summary>
		/// Database unique identifier for this SPRole
		/// </summary>
		public int ID{get{return _ID;}set{_ID=value;}}

		/// <summary>
		/// Role Name.
		/// </summary>
		public string Name{get{return _Name;}set{_Name=value;}}

		/// <summary>
		/// Role Description.
		/// </summary>
		public string Description{get{return _Description;}set{_Description=value;}}

		/// <summary>
		/// ArrayList of UserInfo structs, containing User LoginName, email, etc.
		/// </summary>
		public ArrayList Users{get{return _Users;}set{_Users=value;}}

		/// <summary>
		/// True if the Role name requested in the constructor actually exists in the site (i.e, has been 
		/// persisted in the database).
		/// </summary>
		public bool Exists {get{return _Exists;}}

		}
}
