/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using System.Collections;
using System.Configuration;

namespace PREPWorkflow
{
	/// <summary>
	/// Class which represents a PREP Task
	/// </summary>
	public class PREPTask
	{
		private string _Title;
		private string _AssignedTo;
		private string _Status;
		private DateTime _StartDate;
		private DateTime _DueDate;
		private string _DocumentName;
		private string _DocumentAuthor;		
		private string _AssignmentName;
		private SPListItem _SPListItemObject;
		private bool _Exists;

		/// <summary>
		/// Default constructor
		/// </summary>
		public PREPTask()
		{}

		/// <summary>
		/// Populates a PREPTask object from a a SharePoint List item, taken from the PREP Task list.
		/// </summary>
		/// <param name="listItem">The SharePoint SPListItem object from which to construct this PREPTask object.</param>
		public PREPTask(SPListItem listItem)
		{
			try
			{
				this._Title = (string)listItem["Title"];
				// the following code won't work because you can't cast the "Assigned To" field to SPUser, even though that is its type
				//SPUser assignedTo = (SPUser)listItem["Assigned To"];

				//the following code won't work because sometimes the user name in "Assigned To" is the display name and sometimes
				//it is the login. The field is actually of type string with a value something like "1;#DOMAIN\username"
				//thisTask.AssignedTo = Utilities.GetUserNameFromListItemField(listItem, "Assigned To");
				//the "AssignedTo Login" field was created as a workaround
				this._AssignedTo = (string)listItem["AssignedTo Login"];
				this._Status = (string)listItem["Status"];
				// The date fields might be empty if a task was added to this list by a user. In that case, ignore the error.
				try{this._StartDate = (DateTime)listItem["Start Date"];}
				catch{}
				try{this._DueDate = (DateTime)listItem["Due Date"];}
				catch{}
				this._DocumentName = (string)listItem["Document Name"];
				this._DocumentAuthor = (string)listItem["Document Author"];
				this._AssignmentName = (string)listItem["Assignment Name"];
				this._SPListItemObject = listItem;
				this._Exists = true;
			}
			catch (Exception ex)
			{
				Utilities.LogError(listItem.ParentList.ParentWeb, "Error in PREPTask(SPListItem listItem) ", ex);
			}
		}

		/// <summary>
		/// A Collection of PREPTask objects containing all of the PREP Tasks for a site
		/// </summary>
		/// <param name="prepTaskList">The SharePoint list object for the PREP Review Task list.</param>
		/// <returns>An ArrayList of PREPTask objects.</returns>
		public static ArrayList PREPTask_Collection(SPList prepTaskList)
		{
			ArrayList prepTasks = new ArrayList();
			try
			{
				SPListItemCollection taskCollection = prepTaskList.Items;
							
				foreach (SPListItem listItem in taskCollection)
				{
					PREPTask thisTask = new PREPTask(listItem);
					prepTasks.Add(thisTask);
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError(prepTaskList.ParentWeb, "Error in PREPTask_Collection. " , ex);
			}
			return prepTasks;
		}

		/// <summary>
		/// A Collection of PREPTask objects containing all of the PREP Tasks for a given assignment
		/// </summary>
		/// <param name="assignmentName">The assignment to return tasks for</param>
		/// <param name="prepTaskList">The SharePoint list object for the PREP Review Task list.</param>
		/// <returns>An ArrayList of PREPTask objects.</returns>
		public static ArrayList PREPTask_Collection(SPList prepTaskList, string assignmentName)
		{
			ArrayList prepTasks = new ArrayList();
			try
			{
				// make sure it's a real assignment name. If it is, there will be a role for it.
				PREPRole role = new PREPRole(prepTaskList.ParentWeb, assignmentName);
				if(role.Exists)
				{
					SPListItemCollection taskCollection = prepTaskList.Items;
							
					foreach (SPListItem listItem in taskCollection)
					{
						string taskAssignmentName = (string)listItem["Assignment Name"];
						if(taskAssignmentName != null && taskAssignmentName != "")
						{
							// We're only interested in the passed-in assignment name.
							if(taskAssignmentName == assignmentName)
							{
							
								PREPTask thisTask = new PREPTask(listItem);
								prepTasks.Add(thisTask);
							}
						}
					} //foreach (SPListItem listItem in taskCollection)
				} // if(role.Exists)
			} //try
			catch (Exception ex)
			{
				Utilities.LogError(prepTaskList.ParentWeb, "PREPTask_Collection(SPList prepTaskList, string assignmentName). assignmentName=" +assignmentName, ex);
			}
			return prepTasks;
		}

		/// <summary>
		/// Returns a user's (represented as userLoginName) "ideas" task - 
		/// a task obligating them to create and upload an ideas document.
		/// If no such task exists, the boolean "Exists" is set to false.
		/// </summary>
		/// <param name="prepTasks">ArrayList of PREPTask objects</param>
		/// <param name="assignmentName">The assignment name to check</param>
		/// <param name="userLoginName">The user whose ideas task will be sought.</param>
		/// <returns></returns>
		public static PREPTask GetIdeasTask(ArrayList prepTasks, string assignmentName, string userLoginName)
		{
			PREPTask resultTask = new PREPTask();
			resultTask.Exists = false;
			try
			{
				foreach(PREPTask task in prepTasks)
				{
					//if this task pertains to the current assignment
					if (task.AssignmentName == assignmentName)
					{
						// if this task is assigned to the passed-in user
						if(task.AssignedTo == userLoginName)
						{
							// An ideas task is defined as one where the document author (of the ideas) 
							// is the same person as the person the task is assigned to.
							if(task.DocumentAuthor == task.AssignedTo)
							{
								resultTask = task;
								resultTask.Exists = true;
								return resultTask;
							}
						}
					}
				} //foreach(PREPTask task in prepTasks)
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in PREPTask.GetIdeasTask", ex);
			}
			return resultTask;
		}

		/// <summary>
		/// Returns a "review" task directing userLoginName to review ideasAuthorLoginName's ideas document.
		/// Boolean "Exists" is set to false if no such task exists.
		/// </summary>
		/// <param name="prepTasks">An ArrayList of PREPTask objects to search for a task.</param>
		/// <param name="assignmentName">The name of the assignment whose tasks are to be queried.</param>
		/// <param name="ideasAuthorLoginName">The login name of the author of the ideas document referenced on the task.</param>
		/// <param name="reviewAuthorLoginName">The login name of the user who the task is assigned to.</param>
		/// <returns>A PREPTask object containing a Review Task. If the sought-after task could not be found, this object will have its "Exists" property set to false.</returns>
		public static PREPTask GetReviewTask(ArrayList prepTasks, string assignmentName, string ideasAuthorLoginName, string reviewAuthorLoginName)
		{
			PREPTask resultTask = new PREPTask();
			resultTask.Exists = false;
			try
			{
				foreach(PREPTask task in prepTasks)
				{
					//if this task pertains to the current assignment
					if (task.AssignmentName == assignmentName)
					{
						// if this task is assigned to the passed-in user
						if(task.AssignedTo == reviewAuthorLoginName)
						{
							// if this task pertains to the user named in ideasAuthorLoginName
							if(task.DocumentAuthor == ideasAuthorLoginName)
							{
								// A reviewer does not review his/her own document; if these are the same it cannot be a review task
								if(ideasAuthorLoginName != reviewAuthorLoginName)
								{
									resultTask = task;
									return resultTask;
								}
							}
						}
					}

				} // foreach(PREPTask task in prepTasks)
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in PREPTask.GetReviewTask", ex);
			}
			return resultTask;
		}

		/// <summary>
		/// This method will create a PREP Task, assigned to one user, for the passed in assignmentName. 
		/// The docAuthorLogin is the login of the author of the Ideas Document this task pertains to.
		/// The user is the user who the task is assigned to.
		/// If the docAuthorLogin == the user, this is the user's task to upload an ideas document. 
		/// If the docAuthorLogin != the user, this is the user's task to review the docAuthor's ideas document.
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="PREPList">The PREP Task List.</param>
		/// <param name="assignmentName">The name of the PREP assignment.</param>
		/// <param name="docName">The name of the Ideas Document associated with a PREP task.</param>
		/// <param name="docAuthorLoginName">The login name of the user whose Ideas Document this task pertains to.</param>
		/// <param name="user">The user this task is assigned to.</param>
		public static void Create(SPWeb PREPWeb, SPList PREPList, string assignmentName, string docName, string docAuthorLoginName, SPUser user)
		{
			try
			{
				// find the PREPAssignment object associated with the passed-in assignmentName
				PREPAssignment thisPREPAssignment = new PREPAssignment(PREPWeb, assignmentName);
				
				SPQuery q = new SPQuery();
				q.RowLimit = 1;

				string assignedToName = "";
				if(user.Name == "")
				{
					assignedToName = docAuthorLoginName;
				}
				else
				{
					assignedToName = user.Name;
				}
				SPListItem newItem = PREPList.GetItems(q).Add();
				// IDEAS TASK
				// if the ideas author == the user the task is assigned to, this task directs the user to create an ideas document.
				string userLoginName = Utilities.GetLoginName(user.LoginName);
				if(docAuthorLoginName == userLoginName)
				{
					newItem["Title"] = assignedToName + ": Create and upload an ideas document for assignment " + assignmentName + ".";
					newItem["Start Date"] = thisPREPAssignment.AssignedDate;
					newItem["Due Date"] = thisPREPAssignment.IdeasDueDate;
				}
					// REVIEW TASK
					// if the ideas author != the user the task is assigned to, this task directs the user to review the ideas author's document.
				else
				{
					newItem["Title"] = assignedToName + ": Review " + docName + " by " + docAuthorLoginName + " for assignment " + assignmentName + ".";
					newItem["Start Date"] = thisPREPAssignment.IdeasDueDate;
					newItem["Due Date"] = thisPREPAssignment.ReviewsDueDate;
				}
				newItem["Assigned To"] = user;
				newItem["Status"] = PREPFields.NotStarted; //"Not Started";
			
				// custom fields
				newItem["Document Name"] = docName;
				newItem["Document Author"] = docAuthorLoginName;
				newItem["Assignment Name"] = thisPREPAssignment.AssignmentName;
				newItem["AssignedTo Login"] = userLoginName;
			
				// persist the changes to disk
				newItem.Update();
			}
			catch (Exception ex)
			{
				Utilities.LogError(PREPWeb, "Cannot create PREP task in PREPTask.Create(). assignmentName="+assignmentName + " docAuthorLoginName="+docAuthorLoginName + " user=" + user.LoginName,ex);
			}
		}

		/// <summary>
		/// This will delete all tasks for one user for one assignment. 
		/// This should be run if a user is removed from an assignment.
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="assignmentName">The PREP Assignment name</param>
		/// <param name="currentUserLoginName">Name of the user whose tasks are to be deleted</param>
		public static void DeleteTasksAssignedToUser(SPWeb PREPWeb, string assignmentName, string currentUserLoginName)
		{
			try
			{
				SPList PREPList = PREPWeb.Lists[ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"]];

				// Look for tasks for this user
				
				// Populate an ArrayList of PREP Task objects
				ArrayList prepTasks = PREPTask.PREPTask_Collection(PREPList);
				
				// have to loop backwards because deleting things from the PREPList Items collection disturbs the index when going forward
				for(int i=prepTasks.Count-1; i>-1; i--)
				{
					PREPTask task = (PREPTask)prepTasks[i];
					//if this task pertains to the current assignment
					if (task.AssignmentName == assignmentName)
					{
						// if this task is assigned to the passed-in user
						if(task.AssignedTo == currentUserLoginName)
						{
							//Delete the task
							// The delete method is on the SPListItemCollection object. The index on the 
							// items in this collection should exactly parallel the index on the ArrayList
							// of PREPTask items, since it was built from this collection. 
							// (See PREPTask.PREPTask_Collection)
							// Note: this is why we have to use the PREPTask_Collection method that does not have assignment as an argument.
							// That would result in an ArrayList with a completely different size and index, which would completely throw off
							// this deletion routine, causing complete calamity!
							PREPList.Items.Delete(i);
						} // if(task.AssignedTo == currentUserLoginName)
					} //if (task.AssignmentName == assignmentName)
				} // for loop
			}
			catch (Exception ex)
			{
				Utilities.LogError(PREPWeb, "Cannot delete PREP task in PREPTask.DeleteTasksAssignedToUser(). currentUserLoginName="+currentUserLoginName+", assignmentName="+assignmentName, ex);
			}
		}


		/// <summary>
		/// This will delete all tasks related to a user's Ideas Document for one assignment. 
		/// It will include the user's own task to create an Ideas Document, as well as other 
		/// assignment members tasks to review that Ideas Document. 
		/// This should be run if a user is removed from an assignment.
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="assignmentName">PREP Assignment name for which tasks should be deleted.</param>
		/// <param name="currentUserLoginName">Login Name of the author of the Ideas Document which is the object of the tasks to be deleted.</param>
		public static void DeleteTasksRelatedToUser(SPWeb PREPWeb, string assignmentName, string currentUserLoginName)
		{
			try
			{
				SPList PREPList = PREPWeb.Lists[ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"]];

				// Look for tasks related to this user
				
				// Populate an ArrayList of PREP Task objects
				ArrayList prepTasks = PREPTask.PREPTask_Collection(PREPList);
				
				// have to loop backwards because deleting things from the PREPList Items collection disturbs the index when going forward
				for(int i=prepTasks.Count-1; i>-1; i--)
				{
					PREPTask task = (PREPTask)prepTasks[i];
					//if this task pertains to the current assignment
					if (task.AssignmentName == assignmentName)
					{
						// if this task is related to a document by the passed-in user
						if(task.DocumentAuthor == currentUserLoginName)
						{
							//Delete the task
							// The delete method is on the SPListItemCollection object. The index on the 
							// items in this collection should exactly parallel the index on the ArrayList
							// of PREPTask items, since it was built from this collection. 
							// (See PREPTask.PREPTask_Collection)
							// Note: this is why we have to use the PREPTask_Collection method that does not have assignment as an argument.
							PREPList.Items.Delete(i);
						} // if(task.AssignedTo == currentUserLoginName)
					} //if (task.AssignmentName == assignmentName)
				} // for loop
			}
			catch (Exception ex)
			{
				Utilities.LogError(PREPWeb, "Cannot delete PREP task in PREPTask.DeleteTasksRelatedToUser(). currentUserLoginName="+currentUserLoginName+", assignmentName="+assignmentName, ex);
			}
		}

		/// <summary>
		/// Deletes all PREP Tasks pertaining to a document, meaning the document's name is in the
		/// PREPTask object's DocumentName field. The DocumentName field holds the name of the
		/// Ideas Document the task pertains to. 
		/// The ideas task will be left in place, and its status will be reset to "Not Started".
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context.</param>
		/// <param name="assignmentName">The PREP Assignment name.</param>
		/// <param name="documentName">The PREP Ideas document which is the object of the tasks to be deleted.</param>
		/// <param name="ideasAuthorLoginName">The Login Name of the Author of the PREP Ideas document which is the object of the tasks to be deleted.</param>
		public static void DeleteTasksRelatedToDocument(SPWeb PREPWeb, string assignmentName, string documentName, string ideasAuthorLoginName)
		{
			try
			{
				SPList PREPList = PREPWeb.Lists[ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"]];

				// Look for tasks related to this user
				
				// Populate an ArrayList of PREP Task objects
				ArrayList prepTasks = PREPTask.PREPTask_Collection(PREPList);
				
				// have to loop backwards because deleting things from the PREPList Items collection disturbs the index when going forward
				for(int i=prepTasks.Count-1; i>-1; i--)
				{
					PREPTask task = (PREPTask)prepTasks[i];
					//if this task pertains to the current assignment
					if (task.AssignmentName == assignmentName)
					{
						// if this task is related to the passed in document name
						if(task.DocumentName == documentName)
						{
							// if this task has the passed-in ideasAuthorLoginLoginName
							if(task.DocumentAuthor == ideasAuthorLoginName)
							{
								if(task.AssignedTo == task.DocumentAuthor)
								{
									// this is the ideas task - a user's obligation to upload an ideas document.
									// One of these should always exist, so it will not be deleted.
									// However, the status will be reset to "Not Started"
									task.SPListItemObject["Status"] = PREPFields.NotStarted;
									task.SPListItemObject.Update();
								}
								else
								{
									//Delete the task
									// The delete method is on the SPListItemCollection object. The index on the 
									// items in this collection should exactly parallel the index on the ArrayList
									// of PREPTask items, since it was built from this collection. 
									// (See PREPTask.PREPTask_Collection)
									// Note: this is why we have to use the PREPTask_Collection method that does not have assignment as an argument.
									PREPList.Items.Delete(i);
								}
							} //if(task.DocumentAuthor == ideasAuthorLoginName)
						} // if(task.DocumentName == documentName)
					} // if (task.AssignmentName == assignmentName)
				} // for loop
			}
			catch (Exception ex)
			{
				Utilities.LogError(PREPWeb, "Cannot delete PREP task in PREPTask.DeleteTasksRelatedToDocument(). documentName="+documentName+", assignmentName="+assignmentName, ex);
			}
		}

		/// <summary>
		/// Task title. Contains information about the ideas document author, the assignment name, and the user it is assigned to.
		/// </summary>
		public string Title{get{return _Title;}set{_Title = value;}}
		
		/// <summary>
		/// Login Name of the User the task is assigned to.
		/// </summary>
		public string AssignedTo{get{return _AssignedTo;}set{_AssignedTo = value;}}
		
		/// <summary>
		/// Task Status: Not Started, In Progress, or Completed
		/// </summary>
		public string Status{get{return _Status;}set{_Status = value;}}
		
		/// <summary>
		/// Date the Task is assigned.
		/// </summary>
		public DateTime StartDate{get{return _StartDate;}set{_StartDate = value;}}
		
		/// <summary>
		/// Date the Task is due.
		/// </summary>
		public DateTime DueDate{get{return _DueDate;}set{_DueDate = value;}}
		
		/// <summary>
		/// Ideas Document Name. This field names the Ideas Document, regardless of whether the task is an Ideas Task (an obligation to upload an Ideas Document) or a Review Task (an obligation to Review an Ideas Document).
		/// </summary>
		public string DocumentName{get{return _DocumentName;}set{_DocumentName = value;}}
		
		/// <summary>
		/// Ideas Document Author. This field names the Ideas Document Author, regardless of whether the task is an Ideas Task (an obligation to upload an Ideas Document) or a Review Task (an obligation to Review an Ideas Document).
		/// </summary>
		public string DocumentAuthor{get{return _DocumentAuthor;}set{_DocumentAuthor = value;}}
		
		/// <summary>
		/// The PREP Assignment Name.
		/// </summary>
		public string AssignmentName{get{return _AssignmentName;}set{_AssignmentName = value;}}
		
		/// <summary>
		/// The complete SPList object taken from the SharePoint list. This has all of the properties, as well as the Update() method.
		/// </summary>
		public SPListItem SPListItemObject{get{return _SPListItemObject;}set{_SPListItemObject = value;}}
		
		/// <summary>
		/// Returns true if the constructor for the current instance was able to locate a persisted record corresponding to the requested key.
		/// </summary>
		public bool Exists{get{return _Exists;}set{_Exists = value;}}
	
	}

	/// <summary>
	/// Class which standardizes the string representation of various PREP fields
	/// </summary>
	public class PREPFields
	{
		/// <summary>
		/// Default Constructor
		/// </summary>
		public PREPFields()
		{}
		
		/// <summary>
		/// Used for PREP Tasks.
		/// </summary>
		public const string NotStarted = "Not Started";
		
		/// <summary>
		/// Used for PREP Tasks.
		/// </summary>
		public const string InProgress = "In Progress";

		/// <summary>
		/// Used for PREP Tasks. 
		/// </summary>
		public const string Completed = "Completed";

		/// <summary>
		/// Used for PREP Documents
		/// </summary>
		public const string CheckedOutTo = "Checked Out To";

		/// <summary>
		/// Used for PREP Document metadata
		/// </summary>
		public const string DocTypeIdeas = "Ideas";

		/// <summary>
		/// Used for PREP Document metadata
		/// </summary>
		public const string DocTypeReview = "Review";
	}
}
