/*=====================================================================
  PREPEventSink.cs

  Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
  Please see license.txt in top level directory for full license.

---------------------------------------------------------------------

=====================================================================*/
using System;
using System.Xml;
using System.Threading;
using System.Security.Principal;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using System.Collections;
using System.Configuration;
using System.Text;

namespace PREPWorkflow
{
    /// <summary>
    /// The main class for the PREP Event Handler
    /// </summary>
    public class PREPEventSink : BaseEventSink
    {
		private Guid m_PREPListID = Guid.Empty;
		private WindowsIdentity m_Identity = null;
		private SPList m_list;

		private string prepDocLibraryName = ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"];

		private string prepTaskListName = ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"];

		char[] slash = {'/'};
		char[] backslash = {'\\'};
		char[] pound = {'#'};

		string userID, domain, password;

        /// <summary>
        /// Windows HandlerIdentity
        /// </summary>
		protected override WindowsIdentity HandlerIdentity
        {
            get
            {
                if (m_Identity == null)
                {
                    // TODO: ***** PROVIDE YOUR IMPLEMENTATION HERE ****
                    // Get the Credentials (username / password) that this event sink instance should run as
                    // and Initialize the m_identity object by calling BaseEventSink.CreateIdentity
                    //
                    // m_Identity = CreateIdentity(username, domain, password);   
					try
					{
						// PREPWorkflow.Username, PREPWorkflow.Domain, and PREPWorkflow.Password are stored in machine.config
						userID = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Username"]);
						domain = ConfigurationSettings.AppSettings["PREPWorkflow.Domain"];
						password = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Password"]);
						m_Identity = CreateIdentity(userID, domain, password);
					
					}
					catch (Exception ex)
					{
						// write the credentials to the event log - except keep the password encrypted.
						Utilities.LogError("PREPEventSink cannot impersonate (log in). userID=" + userID + ", domain=" + domain + ", encrypted password=" + ConfigurationSettings.AppSettings["PREPWorkflow.Password"], ex);
					}
                }
                
                return m_Identity;
            }
        }

        /// <summary>
        /// Handle the event from the PREP Document Library
        /// </summary>
        protected override void HandleEvent()
        {
			try
			{
				m_list = GetPREPList();
			}
			catch (Exception ex)
			{
				Utilities.LogError("GetPREPList() failed. ", ex);
			}

			switch (EventInfo.Type)
			{
				case SPListEventType.CheckIn:
					//HandleCheckin();
					break;
				case SPListEventType.CheckOut:
					//HandleCheckout();
					break;
				case SPListEventType.Copy:
					//HandleCopy();
					break;
				case SPListEventType.Delete:
					HandleDelete();
					break;
				case SPListEventType.Insert:
					// Initial add of a document
					HandleInsert();
					break;
				case SPListEventType.Move:
					//HandleMove();
					break;
				case SPListEventType.UncheckOut:
					//HandleUncheckout();
					break;
				case SPListEventType.Update:
					HandleUpdate();
					break;
			}
			m_list = null;
            
        }
        
        private SPList GetPREPList()
        {       
            SPList PREPList = null;

            if (m_PREPListID != Guid.Empty)
            {
                //Get List by ID
                try
                {
                    PREPList = EventWeb.Lists[m_PREPListID];
                }
                catch(SPException)
                {
                    PREPList = CreatePREPList(GetPREPListTitle());
                    m_PREPListID = PREPList.ID;
                }
            }
            else
            {    
                //Get List by name.

                string listName = GetPREPListTitle();
                SPWeb web = EventWeb;
                try
                {
                    PREPList = web.Lists[listName];
                }
                catch(ArgumentException)
                {
                    PREPList = CreatePREPList(listName);
                }

                m_PREPListID = PREPList.ID;
            }
            
            return PREPList;
        }

        /// <summary>
        /// Create a PREP Review list with the correct fields and views.
        /// </summary>
        /// <param name="listName">The name of the log list to create.</param>
        /// <returns>The SPList object corresponding to the created PREPList</returns>
        private SPList CreatePREPList(string listName)
        {           
			SPWeb web = EventWeb;
			Guid id = web.Lists.Add(listName, "Task list for " + EventInfo.Title, SPListTemplateType.Tasks);
			SPList PREPList = web.Lists[id];
        
			// This code is to customize the fields in the list.
			PREPList.Fields.Add("Document Name", SPFieldType.Text, true);
			PREPList.Fields.Add("Document Author", SPFieldType.Text, true);
			PREPList.Fields.Add("Assignment Name", SPFieldType.Text, true);
			PREPList.Fields.Add("AssignedTo Login", SPFieldType.Text, true);

			SPView defView = PREPList.DefaultView;
			SPViewFieldCollection vf = defView.ViewFields;
			vf.Add("Document Name");
			vf.Add("Document Author");
			vf.Add("Assignment Name");
			vf.Add("Assigned To Login");
			vf.Delete("% Complete");
			defView.Update();

            return PREPList;
        }
        private string GetPREPListTitle()
        {
            string listName = null;
            //First try and get it from the SinkData property
            if (EventInfo.SinkData != null && EventInfo.SinkData.Length > 0)
            {
                try
                {
                    XmlDocument xd = new XmlDocument();
                    xd.LoadXml(EventInfo.SinkData);
                    XmlNode xn = xd.SelectSingleNode("//ListName");
                    if (xn != null)
                    {
                        listName = xn.InnerText;
                    }
                }
                catch {}   
            }

            //Then just use the list name from machine.config
            if (listName == null) 
            {
				listName = prepTaskListName;
            }
            return listName;
        }
        
		# region Handlers

		/// <summary>
		/// Each time a new document is uploaded to the document library, a review task
		/// is created for every user on the site except the user who uploaded the document.
		/// </summary>
		private void HandleInsert()
		{
		
			// Get information about the document (file name, author's name, etc.) from the EventInfo object.
			// EventInfo is an SPListEvent object which is instantiated in the BaseEventSink class (part of this project).
			// It contains information about the document that raised the event.

			// Perform a bunch of tests to make sure we belong here:
			
			// Make sure we have a new file, not a new folder
			if(!DocTriggeredEvent())
				return;

			// Make sure we got a valid document
			PREPDocument doc = new PREPDocument(EventWeb, EventInfo);
			if(! doc.Exists)
				return;

			// Make sure the folder structure contains a valid user name and PREP Assignment
			if(! PREPDocument.IsPREPFolder(EventWeb, doc.Folder))
				return;

			// Make sure the current document represents the first upload of this user for this assignment
			if(!PREPDocument.IsNewPREPDocument(EventWeb, doc))
				return;

			// Passed all test; now generate review tasks

			string reviewAuthorLoginName = "";
			string ideasAuthorLoginName;

			PREPTask ideasTask = new PREPTask();
			PREPTask reviewTask = new PREPTask();

			// set the current document's PREP bit to true
			doc.PREP = true;
			doc.SPFileObject.Item["PREP"] = true;

			// determine whether the document is an ideas document or a review document from the folder name.
			// IndexOf: returns the zero-based position of string2 inside of string1. -1 = not found
			// Looking for the User Login of the uploader of this file within the folder name
			// Folder names are in the form PREPDocuments/userLogin/AssignmentName
			if(doc.Folder.ToUpper().IndexOf(doc.EventUserLogin.ToUpper()) > -1)
			{
				//if this doc is in a folder belonging to the uploader, it is an ideas doc
				doc.DocType = PREPFields.DocTypeIdeas;
				doc.SPFileObject.Item["DocType"] = PREPFields.DocTypeIdeas;
			}
			else
			{
				//if this doc is not in a folder belonging to the uploader, it is a review doc
				doc.DocType = PREPFields.DocTypeReview;
				doc.SPFileObject.Item["DocType"] = PREPFields.DocTypeReview;
			}
			//persist the changes
			doc.SPFileObject.Item.Update();
		
			// Get the current project's name and due date
			PREPAssignment curPREPAssignment = new PREPAssignment(EventWeb, doc.AssignmentName);

			// Get the current projects collection of tasks
			ArrayList prepTasks = PREPTask.PREPTask_Collection(m_list, curPREPAssignment.AssignmentName);

	
			switch(doc.DocType)
			{
				// If the uploaded document is an Ideas document, set the "ideas" task to complete and create review tasks
				case PREPFields.DocTypeIdeas:
					ideasTask = PREPTask.GetIdeasTask(prepTasks, curPREPAssignment.AssignmentName, doc.EventUserLogin);
					if(ideasTask.Exists)
					{
						ideasTask.SPListItemObject["Document Name"] = doc.FileName;
						ideasTask.SPListItemObject["Status"] = PREPFields.Completed;
						ideasTask.SPListItemObject.Update();
					}

					// Loop through all users in the current assignment.
					// Create a task for each user except the user who authored the document that 
					// raised the event. 
					// Since this is an Insert, the user who raised the event will also be the document author.
					try
					{
						SPWeb PREPWeb = EventWeb;
			
						SPRole assignmentRole = PREPWeb.Roles[curPREPAssignment.AssignmentName];
						SPUserCollection assignmentUsers = assignmentRole.Users; //PREPWeb.Users;
						foreach (SPUser user in assignmentUsers)
						{
							// The review author is the current user in the loop
							// user name may or may not include a domain component. if it does, strip it off
							reviewAuthorLoginName = Utilities.GetLoginName(user.LoginName);
					
							// The ideas author is the person who just uploaded a document and caused the event.
							ideasAuthorLoginName = doc.EventUserLogin;
							if (ideasAuthorLoginName != reviewAuthorLoginName)
							{
								// Check to see if this user already has a Review task for this assignment. If not, create one.
								reviewTask = PREPTask.GetReviewTask(prepTasks, curPREPAssignment.AssignmentName, ideasAuthorLoginName, reviewAuthorLoginName);
								if(!reviewTask.Exists)
								{
									// Create the task
									PREPTask.Create(EventWeb, m_list, curPREPAssignment.AssignmentName, doc.FileName, ideasAuthorLoginName, user);

									// Now see if there is an existing review document for this task. This could happen if an ideas doc was deleted and reuploaded.
									PREPDocument reviewDoc = PREPDocument.GetReviewDoc(EventWeb, curPREPAssignment.AssignmentName, ideasAuthorLoginName, reviewAuthorLoginName);
									if(reviewDoc.Exists)
									{
										// repopulate prepTasks to pick up the new review task
										prepTasks = PREPTask.PREPTask_Collection(m_list, curPREPAssignment.AssignmentName);
										// get the new review task
										PREPTask newReviewTask = PREPTask.GetReviewTask(prepTasks, curPREPAssignment.AssignmentName, ideasAuthorLoginName, reviewAuthorLoginName);
										// update the new Review task
										newReviewTask.SPListItemObject["Status"] = PREPFields.Completed;
										newReviewTask.SPListItemObject.Update();
									}
								}
							}
						}
					}
					catch (Exception ex)
					{
						Utilities.LogError("Cannot create PREP review tasks in method HandleInsert(). fileName="+ doc.FileName + ", reviewAuthorLoginName="+reviewAuthorLoginName+", EventUserLogin="+doc.EventUserLogin, ex);
					}
					break;

				case PREPFields.DocTypeReview:
					// If the user has uploaded a review document, set their review task to completed.
					reviewTask = PREPTask.GetReviewTask(prepTasks, curPREPAssignment.AssignmentName, doc.IdeasAuthorLogin, doc.EventUserLogin);
					if(reviewTask.Exists)
					{
						reviewTask.SPListItemObject["Status"] = PREPFields.Completed;
						reviewTask.SPListItemObject.Update();
						
						// if this assignment is complete, send an email to the course instructor(s)
						if(PREPAssignment.IsComplete(EventWeb, curPREPAssignment.AssignmentName))
						{
							string subject = "PREP Assignment " + curPREPAssignment.AssignmentName + " complete";
							string message = subject + Environment.NewLine + Environment.NewLine + PREPAssignment.StatusReport(EventWeb, curPREPAssignment.AssignmentName, Environment.NewLine);
							PREPRole.SendMailToMembers(EventWeb, "Instructors", subject, message);
						}

					}

					break;
				default:
					break;
			}

		}

		/// <summary>
		/// When a document is deleted from the PREP Document library, if it is a user's PREP 
		/// ideas document, all tasks related to the document will be deleted.
		/// If any other document is deleted (a review document or a non PREP document),
		/// nothing should happen.
		/// </summary>
		private void HandleDelete()
		{
			try
			{
				// Pull document information from the URL of the file that was deleted
				PREPDocument.DocInfo docInfo = PREPDocument.GetDocInfoFromURL(EventInfo.UrlBefore);

				// Delete review tasks for this document
				if(docInfo.IsValid)
				{
					// make sure the doc is in a valid folder
					if(PREPDocument.IsPREPFolder(EventWeb, "PREP Documents/" + docInfo.IdeasAuthorLoginName + "/" + docInfo.AssignmentName))
					{
						PREPTask.DeleteTasksRelatedToDocument(EventWeb, docInfo.AssignmentName, docInfo.FileName, docInfo.IdeasAuthorLoginName);
					}
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in PREPEventSink.HandleDelete().", ex);
			}
		}

		/// <summary>
		/// This is the routine when a file is checked out.
		/// The status of the PREP task belonging to the user who checked out the 
		/// file is updated to "In Progress".
		/// NOTE: THIS IS NOT BEING USED IN VERSION 2.0
		/// </summary>
		private void HandleCheckout()
		{

			// Get info about the checked out document
			PREPDocument doc = new PREPDocument(EventWeb, EventInfo);

			// Find the task which directed the logged-in user to 
			// download and review the document that was just downloaded. 
			// Update this task's status to "In Progress".

			// This should really be done with an SPQuery object and a CAML query;
			// but I could not get it to work. Every query returned all rows.
			
			SPListItemCollection checkoutTasks = m_list.Items;

			foreach (SPListItem checkoutTask in checkoutTasks)
			{
				// Get the "Assigned To" user's name
				string assignedToLogin = Utilities.GetLoginName((string)checkoutTask["AssignedTo Login"]);
				if(doc.EventUserLogin == assignedToLogin && doc.FileName == (string)checkoutTask["Document Name"])
				{
					// Change the task's status
					checkoutTask["Status"] = PREPFields.InProgress;
			
					// Update the task. Without this line, the change will not be permanent.
					checkoutTask.Update();
				}
			}
			
		}

		/// <summary>
		/// This is the routine when a file is checked back in.
		/// The status of the PREP task belonging to the user who checked out the 
		/// file is updated to "Completed".
		/// NOTE: THIS IS NOT BEING USED IN VERSION 2.0
		/// </summary>
		private void HandleCheckin()
		{
			// Get info about the checked out document
			PREPDocument doc = new PREPDocument(EventWeb, EventInfo);

			// Find the task which directed the logged-in user to 
			// download and review the document that was just downloaded. 
			// Update this task's status to "In Progress".

			SPListItemCollection checkoutTasks = m_list.Items;

			foreach (SPListItem checkoutTask in checkoutTasks)
			{
				// Get the "Assigned To" user's name
				string assignedToLogin = Utilities.GetLoginName((string)checkoutTask["AssignedTo Login"]);
				if(doc.EventUserLogin == assignedToLogin && doc.FileName == (string)checkoutTask["Document Name"])
																								  {
					// Change the task's status
					checkoutTask["Status"] = PREPFields.Completed;
			
					// Update the task. Without this line, the change will not be permanent.
					checkoutTask.Update();
				}
			}

		}

		/// <summary>
		/// This is the routine used when a file that already exists in the library
		/// is re-uploaded.
		/// We will use the Update event to detect and restore PREP-related documents that were 
		/// overwritten by someone other than the author.
		/// SharePoint does not preserve file metadata during an update, and it does not keep it with
		/// past versions. Since our metadata (2 fields - PREP and DocType) is handled by the system and
		/// is never seen by the user (unless they view properties), we will have to restore it here.
		/// </summary>
		private void HandleUpdate()
		{
			try
			{
				
				string updaterLoginName = Utilities.GetLoginName(EventInfo.UserLoginName);
				string adminLoginName = userID = Utilities.Decrypt(ConfigurationSettings.AppSettings["PREPWorkflow.Username"]);
				
				// Perform a series of test to see if we belong here
				
				// Find out who initiated this action. If it was the Administrator, the update occurred elsewhere in this Event Handler
				// and we do not have to do anything else here.
				if(updaterLoginName == adminLoginName)
					return;

				// see if we have a valid PREP Document
				PREPDocument doc = new PREPDocument(EventWeb, EventInfo);
				if(! doc.Exists)
					return;
				
				// Make sure the folder structure contains a valid user name and PREP Assignment
				if(! PREPDocument.IsPREPFolder(EventWeb, doc.Folder))
					return;

				// Continue with update logic...
				
				string originalAuthorLoginName = Utilities.GetLoginName(doc.SPFileObject.Author.LoginName);

				// This struct contains information about the document that just got updated
				PREPDocument.DocInfo updatedDocInfo = PREPDocument.GetDocInfoFromURL(EventInfo.UrlAfter);

				// Next, we need to know if this was the Ideas Document for this folder. Since an update
				// kills the document metadata, we'll get this by going after the Ideas Task for the user
				// represented by the folder name. This value is now in updatedDocInfo.IdeasAuthorLoginName
				ArrayList prepTasks = PREPTask.PREPTask_Collection(m_list, updatedDocInfo.AssignmentName);
				PREPTask ideasTask = PREPTask.GetIdeasTask(prepTasks, updatedDocInfo.AssignmentName, updatedDocInfo.IdeasAuthorLoginName);


				// This will be the ideas document name for the current folder
				string ideasDocName = ideasTask.DocumentName;

				///////// IDEAS DOCUMENT ///////
				// Did the user just update the Ideas Document in the current folder?
				if(ideasDocName == updatedDocInfo.FileName)
				{
					// was this updated by someone other than the original author?
					if(updaterLoginName != originalAuthorLoginName)
					{
						//Restore the last version of the document, plus the metadata
						RestoreDocument(doc, PREPFields.DocTypeIdeas);
					}
					else
					{
						// updated by the author. just restore the metadata.
						// restore the metadata (PREP = true, DocType = Ideas)
						doc.SPFileObject.Item["PREP"] = true;
						doc.SPFileObject.Item["DocType"] = PREPFields.DocTypeIdeas;

						//persist the changes
						doc.SPFileObject.Item.Update();
					}

				} //if(ideasDocName == updatedDocInfo.FileName)
				else //if(ideasDocName == updatedDocInfo.FileName)
				{
					/////// REVIEW DOCUMENT //////
					// If the updated document was not the Ideas Document, see if it was a Review Document.
					// We can't figure this out exactly, so we'll have to do it indirectly.
					
					// Loop through the tasks for the current assignment (loaded above).
					// We should be able to locate a review document for each completed task whose document would 
					// live in the current folder. If this document is not found, an assumption will be made that
					// this is the document that was just updated.

					bool bRestoreDoc = false;
					bool bRestoreMetaData = false;

					foreach(PREPTask task in prepTasks)
					{
						// Only look at tasks that would point to documents in the same folder as the updated document
						if(task.DocumentAuthor == updatedDocInfo.IdeasAuthorLoginName)
						{
							// Only look at review tasks, defined as being assigned to someone other than the ideas author
							if(task.DocumentAuthor != task.AssignedTo)
							{
								// If the task is completed, there should be an existing review document
								if(task.Status == PREPFields.Completed)
								{
									// Normally we use the metadata (PREP = true) to identify a PREP document.
									// However, this usually - but not always - gets obliterated after an update.
									// If the metadata still exists, we'll be able to locate the PREP Review doc
									// Associated with the current task. This is a 100% reliable method of knowing
									// whether or not the update event occurred on the document associated with the
									// current task, since we can compare the event file name with the Review doc file name.
									// If we can't locate the review doc, we're going to assume that there was an update to it
									// and therefore the metadata was obliterated. The very fact that the doc cannot be 
									// identified suggests that something destroyed the metadata. Although it is possible 
									// that a user did this, it is far more likeley to have happened as the result of an 
									// inadvertant update of the review document. At least, that's the best we can do
									// to determine which review document got updated.

									// Get the review document
									PREPDocument reviewDoc = PREPDocument.GetReviewDoc(EventWeb, updatedDocInfo.AssignmentName, updatedDocInfo.IdeasAuthorLoginName, task.AssignedTo);
									if(reviewDoc.Exists)
									{
										// if these filenames match this is definitely an update of this review doc.
										if(doc.FileName == reviewDoc.FileName)
										{
											// was this updated by someone other than the original author?
											if(updaterLoginName != originalAuthorLoginName)
											{
												bRestoreDoc = true;
												break; // got our file; exit the loop
											}
											else
											{
												// updated by the author. just restore the metadata.
												// {PREP = true, DocType = Review)
												bRestoreMetaData = true;
												break; // got our file; exit the loop
											}
										} //if(doc.FileName == reviewDoc.FileName)
									}
									else //(reviewDoc.Exists)
									{
										// The review doc should exist since the task is marked "Completed".
										// If it does not, we will assume that it was updated and its metadata
										// was obliterated.
										// was this updated by someone other than the original author?
										if(updaterLoginName != originalAuthorLoginName)
										{
											bRestoreDoc = true;
											break; // got our file; exit the loop
										}
										else
										{
											// updated by the author. just restore the metadata.
											// {PREP = true, DocType = Review)
											bRestoreMetaData = true;
											break; // got our file; exit the loop
										}
									}
									
								} // if(task.Status == PREPFields.Completed)
							} // (task.DocumentAuthor != task.AssignedTo)
						} // if(task.DocumentAuthor == updatedDocInfo.IdeasAuthorLoginName)
					} // foreach(PREPTask task in prepTasks)
					if(bRestoreDoc)
					{
						// Restore the last version of the document, plus the metadata
						RestoreDocument(doc, PREPFields.DocTypeReview);
					}
					else if(bRestoreMetaData)
					{
						// updated by the author. just restore the metadata.
						// restore the metadata (PREP = true, DocType = Review)
						doc.SPFileObject.Item["PREP"] = true;
						doc.SPFileObject.Item["DocType"] = PREPFields.DocTypeReview;

						//persist the changes
						doc.SPFileObject.Item.Update();
					}
				} // else if(ideasDocName == updatedDocInfo.FileName) [REVIEW DOCUMENT branch]
			} // try
			catch (Exception ex)
			{
				Utilities.LogError(EventWeb, "Error in PREP Event Handler: HandleUpdate()", ex);
			}
		}

		# endregion
		
		#region HelperMethods
		
		/// <summary>
		/// Returns true if there is a document associated with the triggering event. 
		/// Folder creation will also trigger this event, and will return false.
		/// </summary>
		/// <returns>true if a document is associated with the current event.</returns>
		private bool DocTriggeredEvent()
		{
			// try to successfully create a SPFile object.
			try
			{
				SPFolder docLibFolder = EventWeb.GetFolder(EventInfo.UrlAfter.Substring(0, EventInfo.UrlAfter.LastIndexOf("/")));
				SPFileCollection docLibFiles = docLibFolder.Files;
				// Get the file as an SPFile object.
				SPFile thisFile = docLibFiles[EventInfo.UrlAfter];
				return true;
			}
			catch
			{
				return false;
			}
		}

		
		/// <summary>
		/// Does the following: 
		/// 1. Restores a PREP document to the previous version. 
		/// 2. Restores the PREP metadata (fields "PREP" and "DocType")
		///    (this is necessary because SharePoint overwrites metadata from user input
		///    and PREP metadata is hidden from the user)
		/// 3. Emails the user who attempted an update. 
		/// </summary>
		/// <param name="doc">A document object</param>
		/// <param name="docType">Possible values: "Ideas" or "Review" </param>
		private void RestoreDocument(PREPDocument doc, string docType)
		{
			try
			{
				if(docType != PREPFields.DocTypeIdeas && docType != PREPFields.DocTypeReview)
					throw new Exception("docType must be either " + PREPFields.DocTypeIdeas + " or " + PREPFields.DocTypeReview + ".");

			
				// 1. Restore the last version of the file.
				// 2. Restore the metadata
				// 3. Email the user who attempted the update.

				// We have to create two SPWeb objects to update the file.
				// This is to prevent a "Save Conflict" exception where SharePoint
				// will not allow 2 updates within the same process.
				// (Update 1 = restore the last version; Update 2 = reset the metadata)
				// SPWeb implements IDisposable, which means we can put it in a "using" block.
				// This will dispose the objects and avoid the conflict.
				SPWeb updateWeb1 = EventWeb;
				SPWeb updateWeb2 = EventWeb;

				// 1. restore the previous version of the file
				// Use Count-2 to sync 1-based count with 0-based index
				using(updateWeb1)
				{
					SPFile updateFile1 = updateWeb1.GetFile(EventInfo.UrlAfter);
					updateFile1.Versions.Restore(updateFile1.Versions.Count -2);
				}
						
				// 2. restore the metadata (PREP = true, DocType = Ideas)
				using(updateWeb2)
				{
					SPFile updateFile2 = updateWeb2.GetFile(EventInfo.UrlAfter);
					updateFile2.Item["PREP"] = true;
					updateFile2.Item["DocType"] = docType;
					updateFile2.Item.Update();
				}

				// 3. Email the updater
				try
				{
					StringBuilder msg = new StringBuilder();
					msg.Append("File \"" + doc.FileName + "\" cannot be uploaded to SharePoint site \"" + EventWeb.Title + "\" (" + EventWeb.Url + ")" + " in folder \"");
					msg.Append(doc.Folder + "\" because it would overwrite the " + docType + " Document in that folder. Updates to the " + docType + " Document may only be done by the original author of the file. " + Environment.NewLine + Environment.NewLine);
					msg.Append("Please rename your file and upload it again." + Environment.NewLine + Environment.NewLine);
					msg.Append("This is a system-generated message.");
					Utilities.SendEmail("PREP", msg.ToString(), doc.SPFileObject.ModifiedBy.Email);
					Utilities.LogError(EventWeb, "Real User: " + Utilities.GetLoginName(EventInfo.UserLoginName) + Environment.NewLine + Environment.NewLine + msg.ToString() + Environment.NewLine, new Exception()); // !@# Eventually should delete this notification
				}
				catch(Exception ex)
				{
					Utilities.LogError(EventWeb, "Email error in HandleUpdate()", ex);
				}
			}
			catch(Exception ex)
			{
				Utilities.LogError(EventWeb, "Error in RestoreDocument(). Document = " + doc.FileName, ex);
			}
		}

		#endregion
        
    }
	
}
