/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Web.UI;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;
//
using System.Configuration;
using System.Text;
using System.Collections;

namespace PREPWorkflow
{
	/// <summary>
	/// This handler intercepts clicks from the PREPWebPartMatrix Web Part.
	/// Depending on the context (user and document), it will download or upload a document.
	/// </summary>
	public class PREPUploadDownloadHandler : IHttpHandler
	{
		private bool reuseHandler;

		/// <summary>
		/// Default Constructor.
		/// </summary>
		public PREPUploadDownloadHandler()
		{
			reuseHandler = true;
		}
		/// <summary>
		/// Process the HTTP request called by the application process
		/// Part of IHttpHandler.
		/// </summary>
		/// <param name="context"></param>
		public void ProcessRequest(HttpContext context)
		{
			string fileName = "";
			string ideasAuthorName = "";
			string assignmentName = "";
			string action = "";

			try
			{
				if(context.Request["fileName"] != null)
				{
					fileName = context.Request["fileName"].ToString();
				}
				if(context.Request["assignmentName"] != null)
				{
					assignmentName = context.Request["assignmentName"].ToString();
				}
				if(context.Request["ideasAuthorName"] != null)
				{
					ideasAuthorName = context.Request["ideasAuthorName"].ToString();
				}
				if(context.Request["prepAction"] != null)
				{
					action = context.Request["prepAction"].ToString();
				}

				SPWeb PREPWeb = SPControl.GetContextWeb(context);
				PREPWeb.AllowUnsafeUpdates = true;

				// Make some checks against the values obtained from the query string 
				// to prevent URL hacking
				PREPRole role = new PREPRole(PREPWeb, assignmentName);
				if(!role.Exists)
				{
					// throw an exception and redirect to the Home Page
					throw new Exception("Assignment name not valid. assignmentName=" + assignmentName);
				}
				else
				{
					// if the assignment is valid, see if the given user belongs to it
					bool bFound = false;
					foreach(SPUserInfo userInfo in role.Users)
					{
						if(ideasAuthorName == Utilities.GetLoginName(userInfo.LoginName))
						{
							bFound = true;
							break;
						}
					}
					if(!bFound)
					{
						// throw an exception and redirect to the Home Page
						throw new Exception("ideasAuthor on URL not authorized for this assignment. ideasAuthor=" + ideasAuthorName);
					}
				}

				// The SPFolders representation of the document library has no spaces, whereas the SPList version does.
				// i.e. SPFolders representation = "PREPDocuments"; SPList representation = "PREP Documents". Hence the Replace function below.
				// This is an artifact of reconstituting a site from a template; it won't happen if you build the library manually.
				string prepDocLibraryName = ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"].Replace(" ", "");

				SPFolder docLibFolder = PREPWeb.Folders[prepDocLibraryName];

				SPFileCollection docLibFiles = docLibFolder.Files;

				// Get the subfolders collection for the PREP Documents library. docLibFolder is defined above.
				SPFolderCollection folderCollection = docLibFolder.SubFolders;

				SPFolder assignmentFolder;
				
				string path = "";
				
				StringBuilder sRedirect;
				string sRedirectURL;

				string loggedInUserName = Utilities.GetLoginName(context.User.Identity.Name);
				
				// Checkout, download, or upload the file. 
				switch(action)
				{
					case "checkout":

						// Not an actual checkout in the Document Library.. just setting the review task to "In Progress".

						// Update the PREP task to "In Progress"
						ArrayList prepTasks = PREPTask.PREPTask_Collection(PREPWeb.Lists[ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"]]);
						PREPTask reviewTask = PREPTask.GetReviewTask(prepTasks, assignmentName, ideasAuthorName, loggedInUserName);
						if(reviewTask.Exists)
						{
							reviewTask.SPListItemObject["Status"] = PREPFields.InProgress;
							reviewTask.SPListItemObject.Update();
						}

						// Download the file
						// In SharePoint, files do not live on the file system of the server... they live in SQL Server.

						assignmentFolder = PREPWeb.GetFolder(prepDocLibraryName + "/" + ideasAuthorName + "/" + assignmentName);
						if (assignmentFolder.Exists)
						{
							if(assignmentFolder.Files[fileName].Exists)
							{
								path = PREPWeb.Url + "/" + assignmentFolder.Files[fileName].Url;
								context.Response.Redirect(path);
							}
						}
						break;

					case "upload":

						// First, check for the existence of the correct folder. If it does not exist, create it.


						// Get the folder for the user who is the ideas author the uploaded document pertains to.
						// If ideas author == logged in user, it is the logged in user's ideas document.
						// If ideas author != logged in user, it is the logged in user's review of the ideas author's ideas.
						SPFolder userFolder;
						
						userFolder = PREPWeb.GetFolder(prepDocLibraryName + "/" + ideasAuthorName);
						if(! userFolder.Exists)
						{
							folderCollection.Add(ideasAuthorName);
						}
						
						assignmentFolder = PREPWeb.GetFolder(prepDocLibraryName + "/" + ideasAuthorName + "/" + assignmentName);
						if (! assignmentFolder.Exists)
						{
							userFolder.SubFolders.Add(assignmentName);
						}

						// Here we'll redirect to the upload page. If they actually upload the file, 
						// the HandleUPdate() method in the event handler PREPEventSink will take care of updating tasks, etc.

						// Begin building the URL to redirect to. This URL contains contains a complex query string
						// whose job it is to handle the next redirect in the series, i.e. after the upload completes.

						// This was very difficult to duplicate. It had to be EXACTLY the same, including case, as the one generated
						// by SharePoint, or it would not work. In some cases, SharePoint's UrlEncode method was used (SPEncode.UrlEncode),
						// and in some cases the generic one (server.UrlEncode) was used. server.Urlencode creates lower-case hex representations
						// (e.g. %2f), whereas SPEncode makes Upper-case ones (%2F). Sometimes SharePoint requires it one way, sometimes the other.
						// The Guid representing the default View had to have brackets, and appear in upper case.
						// Sometimes pieces of the URL needed to be double-URLEncoded, sometimes single URLEncoded, and sometimes 
						// they had to be double-URLEncoded using two different methods.
						
						// obtain the string representation of the guid that identifies the default view of the folder for use in the query string
						SPList prepDocLibraryAsList = PREPWeb.Lists[ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"]];
						SPView prepDocLibraryDefaultView = prepDocLibraryAsList.DefaultView;
						string viewGuid = prepDocLibraryDefaultView.ID.ToString("B").ToUpper(); // this guid needs brackets and hyphens, hence the "B"

						sRedirect = new StringBuilder();
						// First destination URL
						sRedirect.Append(PREPWeb.Url + "/");
						sRedirect.Append(prepDocLibraryName);
						sRedirect.Append("/Forms/Upload.aspx");
						
						// Where we came from (where we would have come from if this was done from the normal UI)
						sRedirect.Append("?RootFolder=");
						
						//URL Encode this part
						// Where we're going after the upload
						sRedirect.Append(context.Server.UrlEncode(assignmentFolder.ServerRelativeUrl));
						
						sRedirect.Append("&Source="); // no encode
						
						sRedirect.Append(SPEncode.UrlEncode(PREPWeb.Url + "/" + userFolder.ParentFolder.Url + "/Forms/AllItems.aspx"));

						sRedirect.Append(SPEncode.UrlEncode("?RootFolder="));

						//double encode
						sRedirect.Append(context.Server.UrlEncode(context.Server.UrlEncode(assignmentFolder.ServerRelativeUrl)));

						//single encode
						sRedirect.Append(SPEncode.UrlEncode("&View="));

						// double encode using two different methods
						sRedirect.Append(context.Server.UrlEncode(SPEncode.UrlEncode(viewGuid)));

						sRedirectURL = sRedirect.ToString();

						context.Response.Redirect(sRedirectURL);
						break;
					
					case "download":
						
						// force a download of the file. 

						assignmentFolder = PREPWeb.GetFolder(prepDocLibraryName + "/" + ideasAuthorName + "/" + assignmentName);
						if (assignmentFolder.Exists)
						{
							if(assignmentFolder.Files[fileName].Exists)
							{
								path = PREPWeb.Url + "/" + assignmentFolder.Files[fileName].Url;
								context.Response.Redirect(path);
							}
						}

						break;
					
					default:
						// throw an exception and redirect to the Home Page
						throw new Exception("Incorrect action. action=" + action);
						//break;
				}

			}
			catch (Exception ex)
			{
				// The "Thread was being aborted." message happens as the result of a redirect, 
				// so for our purposes it is not an exception.
				if(ex.Message != "Thread was being aborted.")
				{

					SPWeb PREPWeb = SPControl.GetContextWeb(context);
					try
					{
						Utilities.LogError(PREPWeb, "Error in PREPUploadDownloadHandler. " + Environment.NewLine + "URL: " + context.Request.Url.AbsoluteUri + Environment.NewLine + "QueryString: " + context.Request.QueryString + Environment.NewLine, ex);
					}
					catch
					{
						Utilities.LogError("Error in PREPUploadDownloadHandler. ", ex);
					}
					context.Response.Redirect(PREPWeb.Url);
				}
			}
            
		}

		/// <summary>
		/// Property to indicate class reuse state
		/// Part of IHttpHandler
		/// </summary>
		public bool IsReusable
		{
			get
			{
				return reuseHandler;
			}
		}
	}
}
