/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using System.Diagnostics;
using System.Web.Mail;
using System.Text;
using Microsoft.SharePoint;
using Encryption;
using System.Security.Cryptography;
using System.Configuration;
using System.Web.Security;


namespace PREPWorkflow
{
	/// <summary>
	/// Utility class containing methods to log errors, send email, 
	/// encrypt and decrypt strings, manipulate strings, etc.
	/// </summary>
	public class Utilities
	{
		
		/// <summary>
		/// Provides encryption keys for values stored in the appSettings section of machine.config.
		/// </summary>
		public static string MachineKey = ConfigurationSettings.AppSettings["PREPWorkflow.MachineKey"];
		
		/// <summary>
		/// Default Constructor.
		/// </summary>
		public Utilities()
		{
		}

		/// <summary>
		/// Generic error logging mechanism. Writes an error to the Event Log,
		/// and dispatches an email.
		/// </summary>
		/// <param name="message">Message which should accompany the exception.</param>
		/// <param name="ex">The exception.</param>
		public static void LogError(string message, Exception ex)
		{
			// Write to the event log, if possible
			try
			{
				EventLog.WriteEntry("PREPEventSink", message + " Message: " + ex.ToString() , EventLogEntryType.Error);
			}
			catch {}

			// Format the exception to be written to the email
			StringBuilder msg = new StringBuilder();
			msg.Append("Timestamp: " + DateTime.Now + Environment.NewLine);
			msg.Append("Server: " + Environment.MachineName + Environment.NewLine);
			msg.Append("User: " + Environment.UserName + Environment.NewLine);
			msg.Append(message + Environment.NewLine);
			msg.Append("Message: " + ex.Message + Environment.NewLine);
			msg.Append("Source: " + ex.Source + Environment.NewLine);
			msg.Append("Method: " + ex.TargetSite + Environment.NewLine);
			msg.Append("Inner Exception: " + ex.InnerException + Environment.NewLine);
			msg.Append("Stack Trace: " + Environment.NewLine);
			msg.Append(ex.StackTrace + Environment.NewLine + Environment.NewLine);
			msg.Append("Full Dump:" + Environment.NewLine);
			msg.Append(ex.ToString() + Environment.NewLine);

			// Send an email
			MailMessage mail = new MailMessage();
			mail.From = ConfigurationSettings.AppSettings["PREPWorkflow.ErrorEmailFrom"];
			mail.To = ConfigurationSettings.AppSettings["PREPWorkflow.ErrorEmailTo"];
			mail.Subject = "Error in PREPEventSink" ;
			mail.Body = msg.ToString();

			SmtpMail.SmtpServer = "127.0.0.1";
			try
			{	
				SmtpMail.Send(mail);

			}
			catch (Exception exSMTP)
			{
				// Trouble sending email...
				// Report detailed SMTP Errors
				string smtpErrorMsg;
				smtpErrorMsg = "SMTP Exception: " + exSMTP.Message;
				//check the InnerException
				if (exSMTP.InnerException != null)
					smtpErrorMsg += "<br>Inner Exceptions:";
				while( exSMTP.InnerException != null )
				{
					smtpErrorMsg += Environment.NewLine +  exSMTP.InnerException.Message;
					exSMTP = exSMTP.InnerException;
				}
				// Write the error to the event log
				EventLog.WriteEntry("PREPEventSink", smtpErrorMsg, EventLogEntryType.Error);
			}
			finally {}

		}

		/// <summary>
		/// Generic error logging mechanism. Writes an error to the Event Log,
		/// and dispatches an email.
		/// This version returns info about the SharePoint web where it originated.
		/// </summary>
		/// <param name="PREPWeb">A SharePoint site object (SPWeb) set to the current site context. This will enable our log to know which virtual SharePoint site originated the exception. </param>
		/// <param name="message">Message which should accompany the exception.</param>
		/// <param name="ex">The exception.</param>
		public static void LogError(SPWeb PREPWeb, string message, Exception ex)
		{
			// Write to the event log, if possible
			try
			{
				EventLog.WriteEntry("PREPEventSink", message + " Message: " + ex.ToString() , EventLogEntryType.Error);
			}
			catch {}

			// Format the exception to be written to the email
			StringBuilder msg = new StringBuilder();
			msg.Append("Timestamp: " + DateTime.Now + Environment.NewLine);
			msg.Append("Server: " + Environment.MachineName + Environment.NewLine);
			msg.Append("Web Title: " + PREPWeb.Title + Environment.NewLine);
			msg.Append("Web URL: " + PREPWeb.Url + Environment.NewLine);
			msg.Append("User: " + Environment.UserName + Environment.NewLine);
			msg.Append(message + Environment.NewLine);
			msg.Append("Message: " + ex.Message + Environment.NewLine);
			msg.Append("Source: " + ex.Source + Environment.NewLine);
			msg.Append("Method: " + ex.TargetSite + Environment.NewLine);
			msg.Append("Inner Exception: " + ex.InnerException + Environment.NewLine);
			msg.Append("Stack Trace: " + Environment.NewLine);
			msg.Append(ex.StackTrace + Environment.NewLine + Environment.NewLine);
			msg.Append("Full Dump:" + Environment.NewLine);
			msg.Append(ex.ToString() + Environment.NewLine);

			// Send an email
			MailMessage mail = new MailMessage();
			mail.From = ConfigurationSettings.AppSettings["PREPWorkflow.ErrorEmailFrom"];
			mail.To = ConfigurationSettings.AppSettings["PREPWorkflow.ErrorEmailTo"];
			mail.Subject = "Error in PREPEventSink" ;
			mail.Body = msg.ToString();

			SmtpMail.SmtpServer = "127.0.0.1";
			try
			{	
				SmtpMail.Send(mail);

			}
			catch (Exception exSMTP)
			{
				// Trouble sending email...
				// Report detailed SMTP Errors
				string smtpErrorMsg;
				smtpErrorMsg = "SMTP Exception: " + exSMTP.Message;
				//check the InnerException
				if (exSMTP.InnerException != null)
					smtpErrorMsg += "<br>Inner Exceptions:";
				while( exSMTP.InnerException != null )
				{
					smtpErrorMsg += Environment.NewLine +  exSMTP.InnerException.Message;
					exSMTP = exSMTP.InnerException;
				}
				// Write the error to the event log
				EventLog.WriteEntry("PREPEventSink", smtpErrorMsg, EventLogEntryType.Error);
			}
			finally {}

		}

		/// <summary>
		/// Returns only the string user name from a User Name field in a list item.
		/// Strips off ";#" sections as well as domain sections.
		/// </summary>
		/// <param name="item">The SharePoint ListItem object</param>
		/// <param name="fieldName">The name of the field on the ListItem object.</param>
		/// <returns></returns>
		public static string GetUserNameFromListItemField(SPListItem item, string fieldName)
		{
			//char[] slash = {'/'};
			char[] backslash = {'\\'};
			char[] pound = {'#'};

			// get the part after the backslash, if a domain user. 
			// A domain user will look like this: "1;#DOMAINNAME\username"
			string[] userNameParts = ((string)item[fieldName]).Split(backslash);
			string userName = userNameParts[userNameParts.Length - 1].ToString();
			// get the part after the # sign, if present.
			// A non-domain user will look like this: "1;#username"
			userNameParts = userName.Split(pound);
			userName = userNameParts[userNameParts.Length - 1].ToString();

			return userName;
		}

		/// <summary>
		/// Strips off the path component of a file name
		/// </summary>
		/// <param name="fileNameIn">A file name complete with path.</param>
		/// <returns>Just the file name</returns>
		public static string GetFileName(string fileNameIn)
		{
			char[] slash = {'/'};
			string[] pathParts = fileNameIn.Split(slash);
			return pathParts[pathParts.Length -1].ToString();
		}

		/// <summary>
		/// Strips off the path domain component of a login name. 
		/// This method is used frequently in order to make sure that comparisons of login names 
		/// do not fail because they are in different formats.
		/// </summary>
		/// <param name="loginNameIn">The SharePoint login name, which typically contains an identifier and a domain component. Example: "1;#DOMAINNAME\username"/ </param>
		/// <returns>Just the login name, e.g. "username".</returns>
		public static string GetLoginName(string loginNameIn)
		{
			string loginName;
			char[] backslash = {'\\'};
			char[] pound = {'#'};
			// get the part after the backslash, if a domain user. 
			// A domain user will look like this: "1;#DOMAINNAME\username"
			string[] loginParts = loginNameIn.Split(backslash);
			loginName = loginParts[loginParts.Length -1].ToString();
			// get the part after the # sign, if present.
			// A non-domain user will look like this: "1;#username"
			loginParts = loginName.Split(pound);
			loginName = loginParts[loginParts.Length -1].ToString();
			// if this field had no login name it might have had domain or SP user components
			// in that case, return a blank string
			if( (loginName.IndexOf("\\")>-1) || (loginName.IndexOf("#")>-1) )
			{
				loginName = "";
			}
			return loginName;
		}

		/// <summary>
		/// Returns the extension of a file.
		/// </summary>
		/// <param name="fileNameIn">The file name in the form "filename.ext". This method
		/// does not expect to see any path components.</param>
		/// <returns>Just the file extension</returns>
		public static string GetFileExt(string fileNameIn)
		{
			char[] dot = {'.'};
			string[] fileParts = fileNameIn.Split(dot);
			string ext = fileParts[fileParts.Length -1].ToString();
			// handle files with no extension
			if(ext == fileNameIn)
			{ext = "";}
			
			return  ext;
		}

		/// <summary>
		/// Encrypts plainTextToEncrypt
		/// Uses a hard-coded IV and Key
		/// </summary>
		/// <param name="plainTextToEncrypt">text to encrypt</param>
		/// <returns>base 64 string of cipher text</returns>
		public static string Encrypt(string plainTextToEncrypt)
		{
			// Hold the cypher text
			byte[] cipherText = null;

			byte[] keyForCrypt = Encoding.ASCII.GetBytes(Utilities.MachineKey.Substring(8, 24));
			byte[] IV = Encoding.ASCII.GetBytes(Utilities.MachineKey.Substring(64, 8));
            
			try
			{	//Try to encrypt.
				//Create the encryptor.
				Encryptor enc = new Encryptor(EncryptionAlgorithm.TripleDes);
				byte[] plainText = Encoding.ASCII.GetBytes(plainTextToEncrypt);
				enc.IV = IV;

				// Perform the encryption.
				cipherText = enc.Encrypt(plainText, keyForCrypt);
			}
			catch(Exception ex)
			{
				throw new Exception("Exception encrypting. " + ex.Message);
                
			}
			return Convert.ToBase64String(cipherText);
		}

		/// <summary>
		/// Decrypts cipherText
		/// Uses a hard-coded IV and Key
		/// </summary>
		/// <param name="cipherText">base 64 string of cipher text to decode</param>
		/// <returns>the original string unencrypted</returns>
		public static string Decrypt(string cipherText)
		{
			byte[] plainText =null;

			byte[] keyForCrypt = Encoding.ASCII.GetBytes(Utilities.MachineKey.Substring(8, 24));
			byte[] IV = Encoding.ASCII.GetBytes(Utilities.MachineKey.Substring(64, 8));
			try
			{
				// Set up the Decryptor object
				Decryptor dec = new Decryptor(EncryptionAlgorithm.TripleDes);

				// Set the Initialization Vector
				dec.IV = IV;

				// Perform the decryption
				plainText =  dec.Decrypt(Convert.FromBase64String(cipherText), keyForCrypt);
			}
			catch(Exception ex)
			{
				throw new Exception("Error decrypting ciphertext", ex);
			}
            
			return Encoding.ASCII.GetString(plainText);
		}

		/// <summary>
		/// Generic email routine. Sends an email message. Handles the mail object and error checking.
		/// </summary>
		/// <param name="subject">Subject of the email.</param>
		/// <param name="message">Body of the email.</param>
		/// <param name="emailAddress">The email address to send a message to. This string can be specified in the form of a list delimited by either commas or semi-colons, which allows for multiple recipients.</param>
		public static void SendEmail(string subject, string message, string emailAddress)
		{

			// Send the message to each address on the list
			MailMessage mail = new MailMessage();
			mail.From = ConfigurationSettings.AppSettings["PREPWorkflow.ServerEmailFrom"];
			mail.To = emailAddress;
			mail.Subject = subject;
			mail.Body = message;
            
			SmtpMail.SmtpServer = "127.0.0.1";
			try
			{
				SmtpMail.Send(mail);
			}
			catch (Exception exSMTP)
			{
				// Trouble sending email...
				// Report detailed SMTP Errors
				string smtpErrorMsg;
				smtpErrorMsg = "SMTP Exception: " + exSMTP.Message;
				//check the InnerException
				if (exSMTP.InnerException != null)
					smtpErrorMsg += Environment.NewLine + "Inner Exceptions:";
				while (exSMTP.InnerException != null)
				{
					smtpErrorMsg += Environment.NewLine + exSMTP.InnerException.Message;
					exSMTP = exSMTP.InnerException;
				}
				// Write the error to the event log
				EventLog.WriteEntry("PREPEventSink", smtpErrorMsg, EventLogEntryType.Error);
			}
			finally { }

		}

		/// <summary>
		/// Returns a random character string.
		/// </summary>
		/// <param name="numCharacters">The number of characters to return. The maximum value for this is 32.</param>
		/// <returns></returns>
		public static string GetRandomString(int numCharacters)
		{
			string randomString = "";
			try
			{
				if(numCharacters > 32)
					throw new Exception("numCharacters cannot be greater than 32");
				
				// 1. generate a random number
				Random rnd = new Random();
				long rndNo = rnd.Next();

				//2. hash the random number 
				string hashed = FormsAuthentication.HashPasswordForStoringInConfigFile(rndNo.ToString(), "sha1");

				//3. get n characters out of this & make this the new string.
				//ends with 32-numCharacters since there are 32 characters in the hashed string.
				randomString = hashed.Substring(rnd.Next(32-numCharacters), numCharacters);
			}
			catch(Exception ex)
			{
				Utilities.LogError("Error in GetRandomString(). numCharacters="+numCharacters.ToString(), ex);
			}
			return randomString;
		}
	
	}
}
